# coding: utf-8

# python libs
from __future__ import absolute_import
import os
import tempfile
import shutil

# salt testing libs
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    patch,
    mock_open,
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    MockCall,
)

# salt libs
from salt.ext import six
import salt.utils.files
from salt import crypt

# third-party libs
try:
    import M2Crypto
    HAS_M2 = True
except ImportError:
    HAS_M2 = False
try:
    from Cryptodome.PublicKey import RSA  # pylint: disable=unused-import
    HAS_PYCRYPTO_RSA = True
except ImportError:
    HAS_PYCRYPTO_RSA = False
if not HAS_PYCRYPTO_RSA:
    try:
        from Crypto.PublicKey import RSA
        HAS_PYCRYPTO_RSA = True
    except ImportError:
        HAS_PYCRYPTO_RSA = False


PRIVKEY_DATA = (
    '-----BEGIN RSA PRIVATE KEY-----\n'
    'MIIEpAIBAAKCAQEA75GR6ZTv5JOv90Vq8tKhKC7YQnhDIo2hM0HVziTEk5R4UQBW\n'
    'a0CKytFMbTONY2msEDwX9iA0x7F5Lgj0X8eD4ZMsYqLzqjWMekLC8bjhxc+EuPo9\n'
    'Dygu3mJ2VgRC7XhlFpmdo5NN8J2E7B/CNB3R4hOcMMZNZdi0xLtFoTfwU61UPfFX\n'
    '14mV2laqLbvDEfQLJhUTDeFFV8EN5Z4H1ttLP3sMXJvc3EvM0JiDVj4l1TWFUHHz\n'
    'eFgCA1Im0lv8i7PFrgW7nyMfK9uDSsUmIp7k6ai4tVzwkTmV5PsriP1ju88Lo3MB\n'
    '4/sUmDv/JmlZ9YyzTO3Po8Uz3Aeq9HJWyBWHAQIDAQABAoIBAGOzBzBYZUWRGOgl\n'
    'IY8QjTT12dY/ymC05GM6gMobjxuD7FZ5d32HDLu/QrknfS3kKlFPUQGDAbQhbbb0\n'
    'zw6VL5NO9mfOPO2W/3FaG1sRgBQcerWonoSSSn8OJwVBHMFLG3a+U1Zh1UvPoiPK\n'
    'S734swIM+zFpNYivGPvOm/muF/waFf8tF/47t1cwt/JGXYQnkG/P7z0vp47Irpsb\n'
    'Yjw7vPe4BnbY6SppSxscW3KoV7GtJLFKIxAXbxsuJMF/rYe3O3w2VKJ1Sug1VDJl\n'
    '/GytwAkSUer84WwP2b07Wn4c5pCnmLslMgXCLkENgi1NnJMhYVOnckxGDZk54hqP\n'
    '9RbLnkkCgYEA/yKuWEvgdzYRYkqpzB0l9ka7Y00CV4Dha9Of6GjQi9i4VCJ/UFVr\n'
    'UlhTo5y0ZzpcDAPcoZf5CFZsD90a/BpQ3YTtdln2MMCL/Kr3QFmetkmDrt+3wYnX\n'
    'sKESfsa2nZdOATRpl1antpwyD4RzsAeOPwBiACj4fkq5iZJBSI0bxrMCgYEA8GFi\n'
    'qAjgKh81/Uai6KWTOW2kX02LEMVRrnZLQ9VPPLGid4KZDDk1/dEfxjjkcyOxX1Ux\n'
    'Klu4W8ZEdZyzPcJrfk7PdopfGOfrhWzkREK9C40H7ou/1jUecq/STPfSOmxh3Y+D\n'
    'ifMNO6z4sQAHx8VaHaxVsJ7SGR/spr0pkZL+NXsCgYEA84rIgBKWB1W+TGRXJzdf\n'
    'yHIGaCjXpm2pQMN3LmP3RrcuZWm0vBt94dHcrR5l+u/zc6iwEDTAjJvqdU4rdyEr\n'
    'tfkwr7v6TNlQB3WvpWanIPyVzfVSNFX/ZWSsAgZvxYjr9ixw6vzWBXOeOb/Gqu7b\n'
    'cvpLkjmJ0wxDhbXtyXKhZA8CgYBZyvcQb+hUs732M4mtQBSD0kohc5TsGdlOQ1AQ\n'
    'McFcmbpnzDghkclyW8jzwdLMk9uxEeDAwuxWE/UEvhlSi6qdzxC+Zifp5NBc0fVe\n'
    '7lMx2mfJGxj5CnSqQLVdHQHB4zSXkAGB6XHbBd0MOUeuvzDPfs2voVQ4IG3FR0oc\n'
    '3/znuwKBgQChZGH3McQcxmLA28aUwOVbWssfXKdDCsiJO+PEXXlL0maO3SbnFn+Q\n'
    'Tyf8oHI5cdP7AbwDSx9bUfRPjg9dKKmATBFr2bn216pjGxK0OjYOCntFTVr0psRB\n'
    'CrKg52Qrq71/2l4V2NLQZU40Dr1bN9V+Ftd9L0pvpCAEAWpIbLXGDw==\n'
    '-----END RSA PRIVATE KEY-----')

PUBKEY_DATA = (
    '-----BEGIN PUBLIC KEY-----\n'
    'MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA75GR6ZTv5JOv90Vq8tKh\n'
    'KC7YQnhDIo2hM0HVziTEk5R4UQBWa0CKytFMbTONY2msEDwX9iA0x7F5Lgj0X8eD\n'
    '4ZMsYqLzqjWMekLC8bjhxc+EuPo9Dygu3mJ2VgRC7XhlFpmdo5NN8J2E7B/CNB3R\n'
    '4hOcMMZNZdi0xLtFoTfwU61UPfFX14mV2laqLbvDEfQLJhUTDeFFV8EN5Z4H1ttL\n'
    'P3sMXJvc3EvM0JiDVj4l1TWFUHHzeFgCA1Im0lv8i7PFrgW7nyMfK9uDSsUmIp7k\n'
    '6ai4tVzwkTmV5PsriP1ju88Lo3MB4/sUmDv/JmlZ9YyzTO3Po8Uz3Aeq9HJWyBWH\n'
    'AQIDAQAB\n'
    '-----END PUBLIC KEY-----')

MSG = b'It\'s me, Mario'

SIG = (
        b'\x07\xf3\xb1\xe7\xdb\x06\xf4_\xe2\xdc\xcb!F\xfb\xbex{W\x1d\xe4E'
        b'\xd3\r\xc5\x90\xca(\x05\x1d\x99\x8b\x1aug\x9f\x95>\x94\x7f\xe3+'
        b'\x12\xfa\x9c\xd4\xb8\x02]\x0e\xa5\xa3LL\xc3\xa2\x8f+\x83Z\x1b\x17'
        b'\xbfT\xd3\xc7\xfd\x0b\xf4\xd7J\xfe^\x86q"I\xa3x\xbc\xd3$\xe9M<\xe1'
        b'\x07\xad\xf2_\x9f\xfa\xf7g(~\xd8\xf5\xe7\xda-\xa3Ko\xfc.\x99\xcf'
        b'\x9b\xb9\xc1U\x97\x82\'\xcb\xc6\x08\xaa\xa0\xe4\xd0\xc1+\xfc\x86'
        b'\r\xe4y\xb1#\xd3\x1dS\x96D28\xc4\xd5\r\xd4\x98\x1a44"\xd7\xc2\xb4'
        b']\xa7\x0f\xa7Db\x85G\x8c\xd6\x94!\x8af1O\xf6g\xd7\x03\xfd\xb3\xbc'
        b'\xce\x9f\xe7\x015\xb8\x1d]AHK\xa0\x14m\xda=O\xa7\xde\xf2\xff\x9b'
        b'\x8e\x83\xc8j\x11\x1a\x98\x85\xde\xc5\x91\x07\x84!\x12^4\xcb\xa8'
        b'\x98\x8a\x8a&#\xb9(#?\x80\x15\x9eW\xb5\x12\xd1\x95S\xf2<G\xeb\xf1'
        b'\x14H\xb2\xc4>\xc3A\xed\x86x~\xcfU\xd5Q\xfe~\x10\xd2\x9b')


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_PYCRYPTO_RSA, 'pycrypto >= 2.6 is not available')
@skipIf(HAS_M2, 'm2crypto is used by salt.crypt if installed')
class CryptTestCase(TestCase):
    def test_gen_keys(self):
        open_priv_wb = MockCall('/keydir{0}keyname.pem'.format(os.sep), 'wb+')
        open_pub_wb = MockCall('/keydir{0}keyname.pub'.format(os.sep), 'wb+')

        with patch.multiple(os, umask=MagicMock(), chmod=MagicMock(),
                            access=MagicMock(return_value=True)):
            with patch('salt.utils.files.fopen', mock_open()) as m_open, \
                    patch('os.path.isfile', return_value=True):
                result = crypt.gen_keys('/keydir', 'keyname', 2048)
                assert result == '/keydir{0}keyname.pem'.format(os.sep), result
                assert open_priv_wb not in m_open.calls
                assert open_pub_wb not in m_open.calls

            with patch('salt.utils.files.fopen', mock_open()) as m_open, \
                    patch('os.path.isfile', return_value=False):
                crypt.gen_keys('/keydir', 'keyname', 2048)
                assert open_priv_wb in m_open.calls
                assert open_pub_wb in m_open.calls

    @patch('os.umask', MagicMock())
    @patch('os.chmod', MagicMock())
    @patch('os.chown', MagicMock(), create=True)
    @patch('os.access', MagicMock(return_value=True))
    def test_gen_keys_with_passphrase(self):
        key_path = os.path.join(os.sep, 'keydir')
        open_priv_wb = MockCall(os.path.join(key_path, 'keyname.pem'), 'wb+')
        open_pub_wb = MockCall(os.path.join(key_path, 'keyname.pub'), 'wb+')

        with patch('salt.utils.files.fopen', mock_open()) as m_open, \
                patch('os.path.isfile', return_value=True):
            self.assertEqual(crypt.gen_keys(key_path, 'keyname', 2048, passphrase='password'), os.path.join(key_path, 'keyname.pem'))
            result = crypt.gen_keys(key_path, 'keyname', 2048,
                                    passphrase='password')
            assert result == os.path.join(key_path, 'keyname.pem'), result
            assert open_priv_wb not in m_open.calls
            assert open_pub_wb not in m_open.calls

        with patch('salt.utils.files.fopen', mock_open()) as m_open, \
                patch('os.path.isfile', return_value=False):
            crypt.gen_keys(key_path, 'keyname', 2048)
            assert open_priv_wb in m_open.calls
            assert open_pub_wb in m_open.calls

    def test_sign_message(self):
        key = RSA.importKey(PRIVKEY_DATA)
        with patch('salt.crypt.get_rsa_key', return_value=key):
            self.assertEqual(SIG, salt.crypt.sign_message('/keydir/keyname.pem', MSG))

    def test_sign_message_with_passphrase(self):
        key = RSA.importKey(PRIVKEY_DATA)
        with patch('salt.crypt.get_rsa_key', return_value=key):
            self.assertEqual(SIG, crypt.sign_message('/keydir/keyname.pem', MSG, passphrase='password'))

    def test_verify_signature(self):
        with patch('salt.utils.files.fopen', mock_open(read_data=PUBKEY_DATA)):
            self.assertTrue(crypt.verify_signature('/keydir/keyname.pub', MSG, SIG))


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_M2, 'm2crypto is not available')
class M2CryptTestCase(TestCase):
    @patch('os.umask', MagicMock())
    @patch('os.chmod', MagicMock())
    @patch('os.access', MagicMock(return_value=True))
    def test_gen_keys(self):
        with patch('M2Crypto.RSA.RSA.save_pem', MagicMock()) as save_pem:
            with patch('M2Crypto.RSA.RSA.save_pub_key', MagicMock()) as save_pub:
                with patch('os.path.isfile', return_value=True):
                    self.assertEqual(crypt.gen_keys('/keydir', 'keyname', 2048),
                                     '/keydir{0}keyname.pem'.format(os.sep))
                    save_pem.assert_not_called()
                    save_pub.assert_not_called()

                with patch('os.path.isfile', return_value=False):
                    self.assertEqual(crypt.gen_keys('/keydir', 'keyname', 2048),
                                     '/keydir{0}keyname.pem'.format(os.sep))
                    save_pem.assert_called_once_with('/keydir{0}keyname.pem'.format(os.sep), cipher=None)
                    save_pub.assert_called_once_with('/keydir{0}keyname.pub'.format(os.sep))

    @patch('os.umask', MagicMock())
    @patch('os.chmod', MagicMock())
    @patch('os.chown', MagicMock())
    @patch('os.access', MagicMock(return_value=True))
    def test_gen_keys_with_passphrase(self):
        with patch('M2Crypto.RSA.RSA.save_pem', MagicMock()) as save_pem:
            with patch('M2Crypto.RSA.RSA.save_pub_key', MagicMock()) as save_pub:
                with patch('os.path.isfile', return_value=True):
                    self.assertEqual(crypt.gen_keys('/keydir', 'keyname', 2048, passphrase='password'),
                                     '/keydir{0}keyname.pem'.format(os.sep))
                    save_pem.assert_not_called()
                    save_pub.assert_not_called()

                with patch('os.path.isfile', return_value=False):
                    self.assertEqual(crypt.gen_keys('/keydir', 'keyname', 2048, passphrase='password'),
                                     '/keydir{0}keyname.pem'.format(os.sep))
                    callback = save_pem.call_args[1]['callback']
                    save_pem.assert_called_once_with('/keydir{0}keyname.pem'.format(os.sep),
                                                     cipher='des_ede3_cbc',
                                                     callback=callback)
                    self.assertEqual(callback(None), b'password')
                    save_pub.assert_called_once_with('/keydir{0}keyname.pub'.format(os.sep))

    def test_sign_message(self):
        key = M2Crypto.RSA.load_key_string(six.b(PRIVKEY_DATA))
        with patch('salt.crypt.get_rsa_key', return_value=key):
            self.assertEqual(SIG, salt.crypt.sign_message('/keydir/keyname.pem', MSG))

    def test_sign_message_with_passphrase(self):
        key = M2Crypto.RSA.load_key_string(six.b(PRIVKEY_DATA))
        with patch('salt.crypt.get_rsa_key', return_value=key):
            self.assertEqual(SIG, crypt.sign_message('/keydir/keyname.pem', MSG, passphrase='password'))

    def test_verify_signature(self):
        with patch('salt.utils.files.fopen', mock_open(read_data=six.b(PUBKEY_DATA))):
            self.assertTrue(crypt.verify_signature('/keydir/keyname.pub', MSG, SIG))

    def test_encrypt_decrypt_bin(self):
        priv_key = M2Crypto.RSA.load_key_string(six.b(PRIVKEY_DATA))
        pub_key = M2Crypto.RSA.load_pub_key_bio(M2Crypto.BIO.MemoryBuffer(six.b(PUBKEY_DATA)))
        encrypted = salt.crypt.private_encrypt(priv_key, b'salt')
        decrypted = salt.crypt.public_decrypt(pub_key, encrypted)
        self.assertEqual(b'salt', decrypted)


class TestBadCryptodomePubKey(TestCase):
    '''
    Test that we can load public keys exported by pycrpytodome<=3.4.6
    '''

    TEST_KEY = (
        '-----BEGIN RSA PUBLIC KEY-----\n'
        'MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAzLtFhsvfbFDFaUgulSEX\n'
        'Gl12XriL1DT78Ef2/u8HHaSMmPie37BLWas/zaHwI6066bIyYQJ/nUCahTaoHM7L\n'
        'GlWc0wOU6zyfpihCRQHil05Y6F+olFBoZuYbFPvtp7/hJx/D7I/0n2o/c7M5i3Y2\n'
        '3sBxAYNooIQHXHUmPQW6C9iu95ylZDW8JQzYy/EI4vCC8yQMdTK8jK1FQV0Sbwny\n'
        'qcMxSyAWDoFbnhh2P2TnO8HOWuUOaXR8ZHOJzVcDl+a6ew+medW090x3K5O1f80D\n'
        '+WjgnG6b2HG7VQpOCfM2GALD/FrxicPilvZ38X1aLhJuwjmVE4LAAv8DVNJXohaO\n'
        'WQIDAQAB\n'
        '-----END RSA PUBLIC KEY-----\n'
    )

    def setUp(self):
        self.test_dir = tempfile.mkdtemp()
        self.key_path = os.path.join(self.test_dir, 'cryptodom-3.4.6.pub')
        with salt.utils.files.fopen(self.key_path, 'wb') as fd:
            fd.write(self.TEST_KEY.encode())

    def tearDown(self):
        shutil.rmtree(self.test_dir)

    @skipIf(not HAS_M2, "Skip when m2crypto is not installed")
    def test_m2_bad_key(self):
        '''
        Load public key with an invalid header using m2crypto and validate it
        '''
        key = salt.crypt.get_rsa_pub_key(self.key_path)
        assert key.check_key() == 1

    @skipIf(HAS_M2, "Skip when m2crypto is installed")
    def test_crypto_bad_key(self):
        '''
        Load public key with an invalid header and validate it without m2crypto
        '''
        key = salt.crypt.get_rsa_pub_key(self.key_path)
        assert key.can_encrypt()


class TestM2CryptoRegression47124(TestCase):

    SIGNATURE = (
        b'w\xac\xfe18o\xeb\xfb\x14+\x9e\xd1\xb7\x7fe}\xec\xd6\xe1P\x9e\xab'
        b'\xb5\x07\xe0\xc1\xfd\xda#\x04Z\x8d\x7f\x0b\x1f}:~\xb2s\x860u\x02N'
        b'\xd4q"\xb7\x86*\x8f\x1f\xd0\x9d\x11\x92\xc5~\xa68\xac>\x12H\xc2%y,'
        b'\xe6\xceU\x1e\xa3?\x0c,\xf0u\xbb\xd0[g_\xdd\x8b\xb0\x95:Y\x18\xa5*'
        b'\x99\xfd\xf3K\x92\x92 ({\xd1\xff\xd9F\xc8\xd6K\x86e\xf9\xa8\xad\xb0z'
        b'\xe3\x9dD\xf5k\x8b_<\xe7\xe7\xec\xf3"\'\xd5\xd2M\xb4\xce\x1a\xe3$'
        b'\x9c\x81\xad\xf9\x11\xf6\xf5>)\xc7\xdd\x03&\xf7\x86@ks\xa6\x05\xc2'
        b'\xd0\xbd\x1a7\xfc\xde\xe6\xb0\xad!\x12#\xc86Y\xea\xc5\xe3\xe2\xb3'
        b'\xc9\xaf\xfa\x0c\xf2?\xbf\x93w\x18\x9e\x0b\xa2a\x10:M\x05\x89\xe2W.Q'
        b'\xe8;yGT\xb1\xf2\xc6A\xd2\xc4\xbeN\xb3\xcfS\xaf\x03f\xe2\xb4)\xe7\xf6'
        b'\xdbs\xd0Z}8\xa4\xd2\x1fW*\xe6\x1c"\x8b\xd0\x18w\xb9\x7f\x9e\x96\xa3'
        b'\xd9v\xf7\x833\x8e\x01'
    )

    @skipIf(not HAS_M2, "Skip when m2crypto is not installed")
    def test_m2crypto_verify_bytes(self):
        message = salt.utils.stringutils.to_unicode('meh')
        with patch('salt.utils.files.fopen', mock_open(read_data=six.b(PUBKEY_DATA))):
            salt.crypt.verify_signature('/keydir/keyname.pub', message, self.SIGNATURE)

    @skipIf(not HAS_M2, "Skip when m2crypto is not installed")
    def test_m2crypto_verify_unicode(self):
        message = salt.utils.stringutils.to_bytes('meh')
        with patch('salt.utils.files.fopen', mock_open(read_data=six.b(PUBKEY_DATA))):
            salt.crypt.verify_signature('/keydir/keyname.pub', message, self.SIGNATURE)

    @skipIf(not HAS_M2, "Skip when m2crypto is not installed")
    def test_m2crypto_sign_bytes(self):
        message = salt.utils.stringutils.to_unicode('meh')
        key = M2Crypto.RSA.load_key_string(six.b(PRIVKEY_DATA))
        with patch('salt.crypt.get_rsa_key', return_value=key):
            signature = salt.crypt.sign_message('/keydir/keyname.pem', message, passphrase='password')
        self.assertEqual(signature, self.SIGNATURE)

    @skipIf(not HAS_M2, "Skip when m2crypto is not installed")
    def test_m2crypto_sign_unicode(self):
        message = salt.utils.stringutils.to_bytes('meh')
        key = M2Crypto.RSA.load_key_string(six.b(PRIVKEY_DATA))
        with patch('salt.crypt.get_rsa_key', return_value=key):
            signature = salt.crypt.sign_message('/keydir/keyname.pem', message, passphrase='password')
        self.assertEqual(signature, self.SIGNATURE)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
'''

# Import Python Libs
from __future__ import absolute_import, unicode_literals, print_function
import textwrap

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.utils.json
import salt.modules.npm as npm
from salt.exceptions import CommandExecutionError


@skipIf(NO_MOCK, NO_MOCK_REASON)
class NpmTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.npm
    '''
    def setup_loader_modules(self):
        patcher = patch('salt.modules.npm._check_valid_version',
                        MagicMock(return_value=True))
        patcher.start()
        self.addCleanup(patcher.stop)
        return {npm: {}}

    # 'install' function tests: 4

    def test_install(self):
        '''
        Test if it installs an NPM package.
        '''
        mock = MagicMock(return_value={'retcode': 1, 'stderr': 'error'})
        with patch.dict(npm.__salt__, {'cmd.run_all': mock}):
            self.assertRaises(CommandExecutionError, npm.install,
                              'coffee-script')

        # This is at least somewhat closer to the actual output format.
        mock_json_out = textwrap.dedent('''\
        [
          {
            "salt": "SALT"
          }
        ]''')

        # Successful run, expected output format
        mock = MagicMock(return_value={'retcode': 0, 'stderr': '',
                                       'stdout': mock_json_out})
        with patch.dict(npm.__salt__, {'cmd.run_all': mock}):
            self.assertEqual(npm.install('coffee-script'),
                             [{u'salt': u'SALT'}])

        mock_json_out_extra = textwrap.dedent('''\
        Compilation output here

        [bcrypt] Success: "/tmp/node_modules/bcrypt/foo" is installed via remote"
        [grpc] Success: "/usr/lib/node_modules/@foo/bar" is installed via remote"
        [
           {
              "from" : "express@",
              "name" : "express",
              "dependencies" : {
                 "escape-html" : {
                    "from" : "escape-html@~1.0.3",
                    "dependencies" : {},
                    "version" : "1.0.3"
                 }
              },
              "version" : "4.16.3"
           }
        ]''')
        extra_expected = [{u'dependencies':
            {u'escape-html': {
                u'dependencies': {},
                u'from': u'escape-html@~1.0.3',
                u'version': u'1.0.3'}
            },
            u'from': u'express@',
            u'name': u'express',
            u'version': u'4.16.3'}]

        # Successful run, expected output format with additional leading text
        mock = MagicMock(return_value={'retcode': 0, 'stderr': '',
                                       'stdout': mock_json_out_extra})
        with patch.dict(npm.__salt__, {'cmd.run_all': mock}):
            self.assertEqual(npm.install('coffee-script'), extra_expected)

        # Successful run, unexpected output format
        mock = MagicMock(return_value={'retcode': 0, 'stderr': '',
                                       'stdout': 'SALT'})
        with patch.dict(npm.__salt__, {'cmd.run_all': mock}):
            mock_err = MagicMock(side_effect=ValueError())
            # When JSON isn't successfully parsed, return should equal input
            with patch.object(salt.utils.json, 'loads', mock_err):
                self.assertEqual(npm.install('coffee-script'), 'SALT')

    # 'uninstall' function tests: 1

    def test_uninstall(self):
        '''
        Test if it uninstalls an NPM package.
        '''
        mock = MagicMock(return_value={'retcode': 1, 'stderr': 'error'})
        with patch.dict(npm.__salt__, {'cmd.run_all': mock}):
            self.assertFalse(npm.uninstall('coffee-script'))

        mock = MagicMock(return_value={'retcode': 0, 'stderr': ''})
        with patch.dict(npm.__salt__, {'cmd.run_all': mock}):
            self.assertTrue(npm.uninstall('coffee-script'))

    # 'list_' function tests: 1

    def test_list(self):
        '''
        Test if it list installed NPM packages.
        '''
        mock = MagicMock(return_value={'retcode': 1, 'stderr': 'error'})
        with patch.dict(npm.__salt__, {'cmd.run_all': mock}):
            self.assertRaises(CommandExecutionError, npm.list_, 'coffee-script')

        mock = MagicMock(return_value={'retcode': 0, 'stderr': 'error',
                                       'stdout': '{"salt": ["SALT"]}'})
        with patch.dict(npm.__salt__, {'cmd.run_all': mock}):
            mock_err = MagicMock(return_value={'dependencies': 'SALT'})
            with patch.object(salt.utils.json, 'loads', mock_err):
                self.assertEqual(npm.list_('coffee-script'), 'SALT')

    # 'cache_clean' function tests: 1

    def test_cache_clean(self):
        '''
        Test if it cleans the cached NPM packages.
        '''
        mock = MagicMock(return_value={'retcode': 1, 'stderr': 'error'})
        with patch.dict(npm.__salt__, {'cmd.run_all': mock}):
            self.assertFalse(npm.cache_clean())

        mock = MagicMock(return_value={'retcode': 0})
        with patch.dict(npm.__salt__, {'cmd.run_all': mock}):
            self.assertTrue(npm.cache_clean())

        mock = MagicMock(return_value={'retcode': 0})
        with patch.dict(npm.__salt__, {'cmd.run_all': mock}):
            self.assertTrue(npm.cache_clean('coffee-script'))

    # 'cache_list' function tests: 1

    def test_cache_list(self):
        '''
        Test if it lists the NPM cache.
        '''
        mock = MagicMock(return_value={'retcode': 1, 'stderr': 'error'})
        with patch.dict(npm.__salt__, {'cmd.run_all': mock}):
            self.assertRaises(CommandExecutionError, npm.cache_list)

        mock = MagicMock(return_value={'retcode': 0, 'stderr': 'error',
                                       'stdout': ['~/.npm']})
        with patch.dict(npm.__salt__, {'cmd.run_all': mock}):
            self.assertEqual(npm.cache_list(), ['~/.npm'])

        mock = MagicMock(return_value={'retcode': 0, 'stderr': 'error',
                                       'stdout': ''})
        with patch.dict(npm.__salt__, {'cmd.run_all': mock}):
            self.assertEqual(npm.cache_list('coffee-script'), '')

    # 'cache_path' function tests: 1

    def test_cache_path(self):
        '''
        Test if it prints the NPM cache path.
        '''
        mock = MagicMock(return_value={'retcode': 1, 'stderr': 'error'})
        with patch.dict(npm.__salt__, {'cmd.run_all': mock}):
            self.assertEqual(npm.cache_path(), 'error')

        mock = MagicMock(return_value={'retcode': 0, 'stderr': 'error',
                                       'stdout': '/User/salt/.npm'})
        with patch.dict(npm.__salt__, {'cmd.run_all': mock}):
            self.assertEqual(npm.cache_path(), '/User/salt/.npm')



# -*- coding: utf-8 -*-
'''
    :codeauthor: Rupesh Tare <rupesht@saltstack.com>
'''

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals
import os

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.modules.qemu_img as qemu_img


@skipIf(NO_MOCK, NO_MOCK_REASON)
class QemuimgTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.qemu_img
    '''
    def setup_loader_modules(self):
        return {qemu_img: {}}

    def test_make_image(self):
        '''
        Test for create a blank virtual machine image file
        of the specified size in megabytes
        '''
        with patch.object(os.path, 'isabs',
                          MagicMock(side_effect=[False, True, True, True])):
            self.assertEqual(qemu_img.make_image('location', 'size', 'fmt'), '')

            with patch.object(os.path, 'isdir',
                              MagicMock(side_effect=[False, True, True])):
                self.assertEqual(qemu_img.make_image('location', 'size', 'fmt'), '')

                with patch.dict(qemu_img.__salt__,
                                {'cmd.retcode': MagicMock(side_effect=[False, True])}):
                    self.assertEqual(qemu_img.make_image('location', 'size', 'fmt'), 'location')
                    self.assertEqual(qemu_img.make_image('location', 'size', 'fmt'), '')



# -*- coding: utf-8 -*-
'''
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
'''

# Import Python Libs
from __future__ import absolute_import, unicode_literals, print_function
import sys

# Import Salt Testing Libs
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    patch,
    mock_open,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.modules.pam as pam

MOCK_FILE = 'ok ok ignore '


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(sys.platform.startswith('openbsd'), 'OpenBSD does not use PAM')
class PamTestCase(TestCase):
    '''
    Test cases for salt.modules.pam
    '''
    # 'read_file' function tests: 1

    def test_read_file(self):
        '''
        Test if the parsing function works
        '''
        with patch('os.path.exists', return_value=True), \
                patch('salt.utils.files.fopen', mock_open(read_data=MOCK_FILE)):
            self.assertListEqual(pam.read_file('/etc/pam.d/login'),
                                 [{'arguments': [], 'control_flag': 'ok',
                                   'interface': 'ok', 'module': 'ignore'}])



# -*- coding: utf-8 -*-
'''
Tests for the spm files utility
'''
# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import os
import shutil

# Import Salt Testing libs
from tests.support.case import SPMCase
from tests.support.helpers import destructiveTest


@destructiveTest
class SPMFilesTest(SPMCase):
    '''
    Validate the spm files command
    '''
    def setUp(self):
        self.config = self._spm_config()
        self._spm_build_files(self.config)

    def test_spm_files(self):
        '''
        test spm files
        '''
        self._spm_create_update_repo(self.config)
        install = self.run_spm('install', self.config, 'apache')
        get_files = self.run_spm('files', self.config, 'apache')

        os.path.exists(os.path.join(self.config['formula_path'], 'apache',
                                    'apache.sls'))

    def tearDown(self):
        shutil.rmtree(self._tmp_spm)



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Libs
import salt.modules.proxy as proxy

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch,
    call
)


@skipIf(NO_MOCK, NO_MOCK_REASON)
class ProxyTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.proxy
    '''
    def setup_loader_modules(self):
        return {proxy: {'__grains__': {'os': 'Darwin'}}}

    def test_get_http_proxy_macos(self):
        '''
            Test to make sure that we correctly get the current proxy info
            on macOS
        '''
        mock = MagicMock(return_value='Enabled: Yes\nServer: 192.168.0.1\nPort: 3128\nAuthenticated Proxy Enabled: 0')
        expected = {
            'enabled': True,
            'server': '192.168.0.1',
            'port': '3128'
        }

        with patch.dict(proxy.__salt__, {'cmd.run': mock}):
            out = proxy.get_http_proxy()
            mock.assert_called_once_with('networksetup -getwebproxy Ethernet')
            self.assertEqual(expected, out)

    def test_get_https_proxy_macos(self):
        '''
            Test to make sure that we correctly get the current proxy info
            on macOS
        '''
        mock = MagicMock(return_value='Enabled: Yes\nServer: 192.168.0.1\nPort: 3128\nAuthenticated Proxy Enabled: 0')
        expected = {
            'enabled': True,
            'server': '192.168.0.1',
            'port': '3128'
        }

        with patch.dict(proxy.__salt__, {'cmd.run': mock}):
            out = proxy.get_https_proxy()
            mock.assert_called_once_with('networksetup -getsecurewebproxy Ethernet')
            self.assertEqual(expected, out)

    def test_get_ftp_proxy_macos(self):
        '''
            Test to make sure that we correctly get the current proxy info
            on macOS
        '''
        mock = MagicMock(return_value='Enabled: Yes\nServer: 192.168.0.1\nPort: 3128\nAuthenticated Proxy Enabled: 0')
        expected = {
            'enabled': True,
            'server': '192.168.0.1',
            'port': '3128'
        }

        with patch.dict(proxy.__salt__, {'cmd.run': mock}):
            out = proxy.get_ftp_proxy()
            mock.assert_called_once_with('networksetup -getftpproxy Ethernet')
            self.assertEqual(expected, out)

    def test_get_http_proxy_macos_none(self):
        '''
            Test to make sure that we correctly return when theres no proxy set
        '''
        mock = MagicMock(return_value='Enabled: No\nServer:\nPort: 0\nAuthenticated Proxy Enabled: 0')

        with patch.dict(proxy.__salt__, {'cmd.run': mock}):
            out = proxy.get_http_proxy()
            mock.assert_called_once_with('networksetup -getwebproxy Ethernet')
            self.assertEqual({}, out)

    def test_set_http_proxy_macos(self):
        '''
            Test to make sure that we correctly set the proxy info
            on macOS
        '''
        mock = MagicMock()

        with patch.dict(proxy.__salt__, {'cmd.run': mock}):
            out = proxy.set_http_proxy('192.168.0.1', 3128, 'frank', 'badpassw0rd', bypass_hosts='.moo.com,.salt.com')
            mock.assert_called_once_with('networksetup -setwebproxy Ethernet 192.168.0.1 3128 On frank badpassw0rd')
            self.assertTrue(out)

    def test_set_https_proxy_macos(self):
        '''
            Test to make sure that we correctly set the proxy info
            on macOS
        '''
        mock = MagicMock()

        with patch.dict(proxy.__salt__, {'cmd.run': mock}):
            out = proxy.set_https_proxy('192.168.0.1', 3128, 'frank', 'passw0rd', bypass_hosts='.moo.com,.salt.com')
            mock.assert_called_once_with('networksetup -setsecurewebproxy Ethernet 192.168.0.1 3128 On frank passw0rd')
            self.assertTrue(out)

    def test_set_ftp_proxy_macos(self):
        '''
            Test to make sure that we correctly set the proxy info
            on macOS
        '''
        mock = MagicMock()

        with patch.dict(proxy.__salt__, {'cmd.run': mock}):
            out = proxy.set_ftp_proxy('192.168.0.1', 3128, 'frank', 'badpassw0rd', bypass_hosts='.moo.com,.salt.com')
            mock.assert_called_once_with('networksetup -setftpproxy Ethernet 192.168.0.1 3128 On frank badpassw0rd')
            self.assertTrue(out)

    def test_get_http_proxy_windows(self):
        '''
        Test to make sure that we correctly get the current proxy info on
        Windows
        '''
        result = {'vdata': 'http=192.168.0.1:3128;https=192.168.0.2:3128;ftp=192.168.0.3:3128'}
        mock = MagicMock(return_value=result)
        expected = {'server': '192.168.0.1',
                    'port': '3128'}
        with patch.dict(proxy.__grains__, {'os': 'Windows'}):
            with patch.dict(proxy.__salt__, {'reg.read_value': mock}):
                out = proxy.get_http_proxy()
                mock.assert_called_once_with(
                    hive='HKEY_CURRENT_USER',
                    key='SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings',
                    vname='ProxyServer')
                self.assertEqual(expected, out)

    def test_get_https_proxy_windows(self):
        '''
        Test to make sure that we correctly get the current proxy info on
        Windows
        '''
        result = {'vdata': 'http=192.168.0.1:3128;https=192.168.0.2:3128;ftp=192.168.0.3:3128'}
        mock = MagicMock(return_value=result)
        expected = {'server': '192.168.0.2',
                    'port': '3128'}
        with patch.dict(proxy.__grains__, {'os': 'Windows'}):
            with patch.dict(proxy.__salt__, {'reg.read_value': mock}):
                out = proxy.get_https_proxy()
                mock.assert_called_once_with(
                    hive='HKEY_CURRENT_USER',
                    key='SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings',
                    vname='ProxyServer')
                self.assertEqual(expected, out)

    def test_get_ftp_proxy_windows(self):
        '''
        Test to make sure that we correctly get the current proxy info on
        Windows
        '''
        result = {'vdata': 'http=192.168.0.1:3128;https=192.168.0.2:3128;ftp=192.168.0.3:3128'}
        mock = MagicMock(return_value=result)
        expected = {'server': '192.168.0.3',
                    'port': '3128'}
        with patch.dict(proxy.__grains__, {'os': 'Windows'}):
            with patch.dict(proxy.__salt__, {'reg.read_value': mock}):
                out = proxy.get_ftp_proxy()
                mock.assert_called_once_with(
                    hive='HKEY_CURRENT_USER',
                    key='SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings',
                    vname='ProxyServer')
                self.assertEqual(expected, out)

    def test_get_all_proxies_macos_fails(self):
        mock = MagicMock()
        with patch.dict(proxy.__salt__, {'reg.read_value': mock}):
            out = proxy.get_proxy_win()
            assert not mock.called
            self.assertEqual(out, None)

    def test_get_all_proxies_windows(self):
        '''
        Test to make sure that we correctly get the current proxy info on
        Windows
        '''
        results = [{'vdata': 'http=192.168.0.1:3128;https=192.168.0.2:3128;ftp=192.168.0.3:3128'},
                   {'vdata': 1}]
        mock = MagicMock(side_effect=results)
        expected = {'enabled': True,
                    'http': {'server': '192.168.0.1',
                             'port': '3128'},
                    'https': {'server': '192.168.0.2',
                              'port': '3128'},
                    'ftp': {'server': '192.168.0.3',
                            'port': '3128'}}
        calls = [
            call(hive='HKEY_CURRENT_USER',
                 key='SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings',
                 vname='ProxyServer'),
            call(hive='HKEY_CURRENT_USER',
                 key='SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings',
                 vname='ProxyEnable')]
        with patch.dict(proxy.__grains__, {'os': 'Windows'}):
            with patch.dict(proxy.__salt__, {'reg.read_value': mock}):
                out = proxy.get_proxy_win()
                mock.assert_has_calls(calls)
                self.assertEqual(expected, out)

    def test_set_http_proxy_windows(self):
        '''
        Test to make sure that we correctly set the proxy info on Windows
        '''
        calls = [
            call(hive='HKEY_CURRENT_USER',
                 key='SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings',
                 vname='ProxyServer',
                 vdata='http=192.168.0.1:3128;'),
            call(hive='HKEY_CURRENT_USER',
                 key='SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings',
                 vname='ProxyEnable',
                 vdata=1,
                 vtype='REG_DWORD'),
            call(hive='HKEY_CURRENT_USER',
                 key='SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings',
                 vname='ProxyOverride',
                 vdata='<local>;.moo.com;.salt.com')]
        mock_reg = MagicMock()
        mock_cmd = MagicMock()
        with patch.dict(proxy.__grains__, {'os': 'Windows'}):
            with patch.dict(proxy.__salt__, {'reg.set_value': mock_reg,
                                             'cmd.run': mock_cmd}):
                out = proxy.set_http_proxy(server='192.168.0.1',
                                           port=3128,
                                           bypass_hosts=['.moo.com', '.salt.com'])
                mock_reg.assert_has_calls(calls)
                mock_cmd.assert_called_once_with('netsh winhttp import proxy source=ie')
                self.assertTrue(out)

    def test_set_https_proxy_windows(self):
        '''
        Test to make sure that we correctly set the proxy info on Windows
        '''
        calls = [
            call(hive='HKEY_CURRENT_USER',
                 key='SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings',
                 vname='ProxyServer',
                 vdata='https=192.168.0.1:3128;'),
            call(hive='HKEY_CURRENT_USER',
                 key='SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings',
                 vname='ProxyEnable',
                 vdata=1,
                 vtype='REG_DWORD'),
            call(hive='HKEY_CURRENT_USER',
                 key='SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings',
                 vname='ProxyOverride',
                 vdata='<local>;.moo.com;.salt.com')]
        mock_reg = MagicMock()
        mock_cmd = MagicMock()
        with patch.dict(proxy.__grains__, {'os': 'Windows'}):
            with patch.dict(proxy.__salt__, {'reg.set_value': mock_reg,
                                             'cmd.run': mock_cmd}):
                out = proxy.set_https_proxy(server='192.168.0.1',
                                            port=3128,
                                            bypass_hosts=['.moo.com', '.salt.com'])
                mock_reg.assert_has_calls(calls)
                mock_cmd.assert_called_once_with('netsh winhttp import proxy source=ie')
                self.assertTrue(out)

    def test_set_ftp_proxy_windows(self):
        '''
        Test to make sure that we correctly set the proxy info on Windows
        '''
        calls = [
            call(hive='HKEY_CURRENT_USER',
                 key='SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings',
                 vname='ProxyServer',
                 vdata='ftp=192.168.0.1:3128;'),
            call(hive='HKEY_CURRENT_USER',
                 key='SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings',
                 vname='ProxyEnable',
                 vdata=1,
                 vtype='REG_DWORD'),
            call(hive='HKEY_CURRENT_USER',
                 key='SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings',
                 vname='ProxyOverride',
                 vdata='<local>;.moo.com;.salt.com')]
        mock_reg = MagicMock()
        mock_cmd = MagicMock()
        with patch.dict(proxy.__grains__, {'os': 'Windows'}):
            with patch.dict(proxy.__salt__, {'reg.set_value': mock_reg,
                                             'cmd.run': mock_cmd}):
                out = proxy.set_ftp_proxy(server='192.168.0.1',
                                          port=3128,
                                          bypass_hosts=['.moo.com', '.salt.com'])
                mock_reg.assert_has_calls(calls)
                mock_cmd.assert_called_once_with('netsh winhttp import proxy source=ie')
                self.assertTrue(out)

    def test_set_proxy_windows(self):
        '''
        Test to make sure that we correctly set the proxy info on Windows
        '''
        calls = [
            call(hive='HKEY_CURRENT_USER',
                 key='SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings',
                 vname='ProxyServer',
                 vdata='http=192.168.0.1:3128;https=192.168.0.1:3128;ftp=192.168.0.1:3128;'),
            call(hive='HKEY_CURRENT_USER',
                 key='SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings',
                 vname='ProxyEnable',
                 vdata=1,
                 vtype='REG_DWORD'),
            call(hive='HKEY_CURRENT_USER',
                 key='SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings',
                 vname='ProxyOverride',
                 vdata='<local>;.moo.com;.salt.com')]
        mock_reg = MagicMock()
        mock_cmd = MagicMock()
        with patch.dict(proxy.__grains__, {'os': 'Windows'}):
            with patch.dict(proxy.__salt__, {'reg.set_value': mock_reg,
                                             'cmd.run': mock_cmd}):
                out = proxy.set_proxy_win(server='192.168.0.1',
                                          port=3128,
                                          bypass_hosts=['.moo.com', '.salt.com'])
                mock_reg.assert_has_calls(calls)
                mock_cmd.assert_called_once_with('netsh winhttp import proxy source=ie')
                self.assertTrue(out)

    def test_set_proxy_windows_no_ftp(self):
        '''
        Test to make sure that we correctly set the proxy info on Windows
        '''
        calls = [
            call(hive='HKEY_CURRENT_USER',
                 key='SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings',
                 vname='ProxyServer',
                 vdata='http=192.168.0.1:3128;https=192.168.0.1:3128;'),
            call(hive='HKEY_CURRENT_USER',
                 key='SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings',
                 vname='ProxyEnable',
                 vdata=1,
                 vtype='REG_DWORD'),
            call(hive='HKEY_CURRENT_USER',
                 key='SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings',
                 vname='ProxyOverride',
                 vdata='<local>;.moo.com;.salt.com')]
        mock_reg = MagicMock()
        mock_cmd = MagicMock()
        with patch.dict(proxy.__grains__, {'os': 'Windows'}):
            with patch.dict(proxy.__salt__, {'reg.set_value': mock_reg,
                                             'cmd.run': mock_cmd}):
                out = proxy.set_proxy_win(server='192.168.0.1',
                                          port=3128,
                                          types=['http', 'https'],
                                          bypass_hosts=['.moo.com', '.salt.com'])
                mock_reg.assert_has_calls(calls)
                mock_cmd.assert_called_once_with('netsh winhttp import proxy source=ie')
                self.assertTrue(out)



# coding: utf-8

# Python libs
from __future__ import absolute_import
from collections import namedtuple

# Salt testing libs
from tests.support.unit import skipIf, TestCase
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, patch, MagicMock, Mock
from tests.support.mixins import LoaderModuleMockMixin

# Salt libs
import salt.beacons.diskusage as diskusage

STUB_DISK_PARTITION = [
    namedtuple(
        'partition',
        'device mountpoint fstype, opts')(
            'tmpfs', '/mnt/tmp', 'tmpfs',
            'rw,nosuid,nodev,relatime,size=10240k'),
    namedtuple(
        'partition',
        'device mountpoint fstype, opts')(
            '/dev/disk0s2', '/', 'hfs',
            'rw,local,rootfs,dovolfs,journaled,multilabel')]

WINDOWS_STUB_DISK_PARTITION = [
    namedtuple(
        'partition',
        'device mountpoint fstype, opts')(
            'C:\\', 'C:\\', 'NTFS',
            'rw,fixed'),
    namedtuple(
        'partition',
        'device mountpoint fstype, opts')(
            'D:\\', 'D:\\', 'CDFS',
            'ro,cdrom')]

STUB_DISK_USAGE = [namedtuple('usage',
                              'total used free percent')(1000, 500, 500, 50),
                   namedtuple('usage',
                              'total used free percent')(100, 75, 25, 25)]

WINDOWS_STUB_DISK_USAGE = namedtuple('usage',
                                     'total used free percent')(1000,
                                                                500,
                                                                500,
                                                                50)


@skipIf(NO_MOCK, NO_MOCK_REASON)
class DiskUsageBeaconTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test case for salt.beacons.adb
    '''

    def setup_loader_modules(self):
        return {}

    def test_non_list_config(self):
        config = {}

        ret = diskusage.validate(config)

        self.assertEqual(ret, (False, 'Configuration for diskusage beacon must'
                                      ' be a list.'))

    def test_empty_config(self):
        config = [{}]

        ret = diskusage.validate(config)

        self.assertEqual(ret, (True, 'Valid beacon configuration'))

    def test_diskusage_match(self):
        disk_usage_mock = Mock(side_effect=STUB_DISK_USAGE)
        with patch('salt.utils.platform.is_windows',
                   MagicMock(return_value=False)),\
                 patch('psutil.disk_partitions',
                       MagicMock(return_value=STUB_DISK_PARTITION)), \
                 patch('psutil.disk_usage', disk_usage_mock):
            config = [{'/': '50%'}]

            ret = diskusage.validate(config)

            self.assertEqual(ret, (True, 'Valid beacon configuration'))

            ret = diskusage.beacon(config)
            self.assertEqual(ret, [{'diskusage': 50, 'mount': '/'}])

    def test_diskusage_nomatch(self):
        disk_usage_mock = Mock(side_effect=STUB_DISK_USAGE)
        with patch('salt.utils.platform.is_windows',
                   MagicMock(return_value=False)),\
                patch('psutil.disk_partitions',
                      MagicMock(return_value=STUB_DISK_PARTITION)), \
                patch('psutil.disk_usage', disk_usage_mock):
            config = [{'/': '70%'}]

            ret = diskusage.validate(config)

            self.assertEqual(ret, (True, 'Valid beacon configuration'))

            ret = diskusage.beacon(config)
            self.assertNotEqual(ret, [{'diskusage': 50, 'mount': '/'}])

    def test_diskusage_match_regex(self):
        disk_usage_mock = Mock(side_effect=STUB_DISK_USAGE)
        with patch('salt.utils.platform.is_windows',
                   MagicMock(return_value=False)),\
                patch('psutil.disk_partitions',
                      MagicMock(return_value=STUB_DISK_PARTITION)), \
                patch('psutil.disk_usage', disk_usage_mock):
            config = [{r'^\/': '50%'}]

            ret = diskusage.validate(config)

            self.assertEqual(ret, (True, 'Valid beacon configuration'))

            ret = diskusage.beacon(config)
            self.assertEqual(ret, [{'diskusage': 50, 'mount': '/'}])

    def test_diskusage_windows(self):
        disk_usage_mock = Mock(return_value=WINDOWS_STUB_DISK_USAGE)
        with patch('salt.utils.platform.is_windows',
                   MagicMock(return_value=True)):
            with patch('psutil.disk_partitions',
                       MagicMock(return_value=WINDOWS_STUB_DISK_PARTITION)), \
                    patch('psutil.disk_usage', disk_usage_mock):
                config = [{'C:\\': '50%'}]

                ret = diskusage.validate(config)

                self.assertEqual(ret, (True, 'Valid beacon configuration'))

                ret = diskusage.beacon(config)
                self.assertEqual(ret, [{'diskusage': 50, 'mount': 'C:\\'}])

    def test_diskusage_windows_match_regex(self):
        disk_usage_mock = Mock(return_value=WINDOWS_STUB_DISK_USAGE)
        with patch('salt.utils.platform.is_windows',
                   MagicMock(return_value=True)):
            with patch('psutil.disk_partitions',
                       MagicMock(return_value=WINDOWS_STUB_DISK_PARTITION)), \
                    patch('psutil.disk_usage', disk_usage_mock):
                config = [{'^[a-zA-Z]:\\': '50%'}]

                ret = diskusage.validate(config)

                self.assertEqual(ret, (True, 'Valid beacon configuration'))

                ret = diskusage.beacon(config)
                _expected = [{u'diskusage': 50, u'mount': 'C:\\'},
                             {u'diskusage': 50, u'mount': 'D:\\'}]
                self.assertEqual(ret, _expected)



# -*- coding: utf-8 -*-
'''
Integration tests for the docker_container states
'''

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals
import functools
import random
import string
import tempfile

# Import Salt Testing Libs
from tests.support.unit import skipIf
from tests.support.case import ModuleCase
from tests.support.paths import TMP
from tests.support.helpers import destructiveTest
from tests.support.mixins import SaltReturnAssertsMixin

# Import Salt Libs
import salt.utils.path

# Import 3rd-party libs
from salt.ext.six.moves import range  # pylint: disable=import-error,redefined-builtin


def _random_name(prefix=''):
    ret = prefix
    for _ in range(8):
        ret += random.choice(string.ascii_lowercase)
    return ret


def with_random_name(func):
    '''
    generate a randomized name for a container
    '''
    @functools.wraps(func)
    def wrapper(self, *args, **kwargs):
        name = _random_name(prefix='salt_')
        return func(self, _random_name(prefix='salt_test_'), *args, **kwargs)
    return wrapper


@destructiveTest
@skipIf(not salt.utils.path.which('dockerd'), 'Docker not installed')
class DockerCallTestCase(ModuleCase, SaltReturnAssertsMixin):
    '''
    Test docker_container states
    '''
    @with_random_name
    def setUp(self, name):
        '''
        setup docker.call tests
        '''
        # Create temp dir
        self.random_name = name
        self.tmp_build_dir = tempfile.mkdtemp(dir=TMP)

        self.run_state('file.managed',
                       source='salt://docker_non_root/Dockerfile',
                       name='{0}/Dockerfile'.format(self.tmp_build_dir))
        self.run_state('docker_image.present',
                       build=self.tmp_build_dir,
                       name=self.random_name)
        self.run_state('docker_container.running',
                       name=self.random_name,
                       image=self.random_name)

    def tearDown(self):
        '''
        teardown docker.call tests
        '''
        self.run_state('file.absent',
                       name=self.tmp_build_dir)
        self.run_state('docker_container.absent',
                       name=self.random_name,
                       force=True)
        self.run_state('docker_image.absent',
                       images=[self.random_name, 'docker.io/opensuse/python:latest'],
                       force=True)

    def test_docker_call(self):
        '''
        check that docker.call works, and works with a container not running as root
        '''
        ret = self.run_function('docker.call', [self.random_name, 'test.ping'])
        self.assertTrue(ret)



# -*- coding: utf-8 -*-
'''
Integration tests for the docker_network states
'''
# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals
import errno
import functools
import logging
import os
import subprocess
import tempfile

# Import Salt Testing Libs
from tests.support.unit import skipIf
from tests.support.case import ModuleCase
from tests.support.docker import with_network, random_name
from tests.support.paths import FILES, TMP
from tests.support.helpers import destructiveTest, requires_system_grains
from tests.support.mixins import SaltReturnAssertsMixin

# Import Salt Libs
import salt.utils.files
import salt.utils.network
import salt.utils.path
from salt.exceptions import CommandExecutionError

log = logging.getLogger(__name__)


IMAGE_NAME = random_name(prefix='salt_busybox_')
IPV6_ENABLED = bool(salt.utils.network.ip_addrs6(include_loopback=True))


def network_name(func):
    '''
    Generate a randomized name for a network and clean it up afterward
    '''
    @functools.wraps(func)
    def wrapper(self, *args, **kwargs):
        name = random_name(prefix='salt_net_')
        try:
            return func(self, name, *args, **kwargs)
        finally:
            self.run_function(
                'docker.disconnect_all_containers_from_network', [name])
            try:
                self.run_function('docker.remove_network', [name])
            except CommandExecutionError as exc:
                if 'No such network' not in exc.__str__():
                    raise
    return wrapper


def container_name(func):
    '''
    Generate a randomized name for a container and clean it up afterward
    '''
    def build_image():
        # Create temp dir
        image_build_rootdir = tempfile.mkdtemp(dir=TMP)
        script_path = \
            os.path.join(FILES, 'file/base/mkimage-busybox-static')
        cmd = [script_path, image_build_rootdir, IMAGE_NAME]
        log.debug('Running \'%s\' to build busybox image', ' '.join(cmd))
        process = subprocess.Popen(
            cmd,
            close_fds=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT)
        output = process.communicate()[0]
        log.debug('Output from mkimge-busybox-static:\n%s', output)

        if process.returncode != 0:
            raise Exception('Failed to build image')

        try:
            salt.utils.files.rm_rf(image_build_rootdir)
        except OSError as exc:
            if exc.errno != errno.ENOENT:
                raise

    @functools.wraps(func)
    def wrapper(self, *args, **kwargs):
        try:
            self.run_function('docker.inspect_image', [IMAGE_NAME])
        except CommandExecutionError:
            pass
        else:
            build_image()

        name = random_name(prefix='salt_test_')
        self.run_function(
            'docker.create',
            name=name,
            image=IMAGE_NAME,
            command='sleep 600',
            start=True)
        try:
            return func(self, name, *args, **kwargs)
        finally:
            try:
                self.run_function('docker.rm', [name], force=True)
            except CommandExecutionError as exc:
                if 'No such container' not in exc.__str__():
                    raise
    return wrapper


@destructiveTest
@skipIf(not salt.utils.path.which('dockerd'), 'Docker not installed')
class DockerNetworkTestCase(ModuleCase, SaltReturnAssertsMixin):
    '''
    Test docker_network states
    '''
    @classmethod
    def tearDownClass(cls):
        '''
        Remove test image if present. Note that this will run a docker rmi even
        if no test which required the image was run.
        '''
        cmd = ['docker', 'rmi', '--force', IMAGE_NAME]
        log.debug('Running \'%s\' to destroy busybox image', ' '.join(cmd))
        process = subprocess.Popen(
            cmd,
            close_fds=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT)
        output = process.communicate()[0]
        log.debug('Output from %s:\n%s', ' '.join(cmd), output)

        if process.returncode != 0 and 'No such image' not in output:
            raise Exception('Failed to destroy image')

    def run_state(self, function, **kwargs):
        ret = super(DockerNetworkTestCase, self).run_state(function, **kwargs)
        log.debug('ret = %s', ret)
        return ret

    @with_network(create=False)
    def test_absent(self, net):
        self.assertSaltTrueReturn(
            self.run_state('docker_network.present', name=net.name))
        ret = self.run_state('docker_network.absent', name=net.name)
        self.assertSaltTrueReturn(ret)
        ret = ret[next(iter(ret))]

        self.assertEqual(ret['changes'], {'removed': True})
        self.assertEqual(
            ret['comment'],
            'Removed network \'{0}\''.format(net.name)
        )

    @container_name
    @with_network(create=False)
    def test_absent_with_disconnected_container(self, net, container_name):
        self.assertSaltTrueReturn(
            self.run_state('docker_network.present',
                name=net.name,
                containers=[container_name])
        )
        ret = self.run_state('docker_network.absent', name=net.name)
        self.assertSaltTrueReturn(ret)
        ret = ret[next(iter(ret))]

        self.assertEqual(
            ret['changes'],
            {
                'removed': True,
                'disconnected': [container_name],
            }
        )
        self.assertEqual(
            ret['comment'],
            'Removed network \'{0}\''.format(net.name)
        )

    @with_network(create=False)
    def test_absent_when_not_present(self, net):
        ret = self.run_state('docker_network.absent', name=net.name)
        self.assertSaltTrueReturn(ret)
        ret = ret[next(iter(ret))]
        self.assertEqual(ret['changes'], {})
        self.assertEqual(
            ret['comment'],
            'Network \'{0}\' already absent'.format(net.name)
        )

    @with_network(create=False)
    def test_present(self, net):
        ret = self.run_state('docker_network.present', name=net.name)
        self.assertSaltTrueReturn(ret)
        ret = ret[next(iter(ret))]

        # Make sure the state return is what we expect
        self.assertEqual(ret['changes'], {'created': True})
        self.assertEqual(
            ret['comment'],
            'Network \'{0}\' created'.format(net.name)
        )

        # Now check to see that the network actually exists. If it doesn't,
        # this next function call will raise an exception.
        self.run_function('docker.inspect_network', [net.name])

    @container_name
    @with_network(create=False)
    def test_present_with_containers(self, net, container_name):
        ret = self.run_state(
            'docker_network.present',
            name=net.name,
            containers=[container_name])
        self.assertSaltTrueReturn(ret)
        ret = ret[next(iter(ret))]

        self.assertEqual(
            ret['changes'],
            {
                'created': True,
                'connected': [container_name],
            }
        )
        self.assertEqual(
            ret['comment'],
            'Network \'{0}\' created'.format(net.name)
        )

        # Now check to see that the network actually exists. If it doesn't,
        # this next function call will raise an exception.
        self.run_function('docker.inspect_network', [net.name])

    def _test_present_reconnect(self, net, container_name, reconnect=True):
        ret = self.run_state(
            'docker_network.present',
            name=net.name,
            driver='bridge')
        self.assertSaltTrueReturn(ret)
        ret = ret[next(iter(ret))]

        self.assertEqual(ret['changes'], {'created': True})
        self.assertEqual(
            ret['comment'],
            'Network \'{0}\' created'.format(net.name)
        )

        # Connect the container
        self.run_function(
            'docker.connect_container_to_network',
            [container_name, net.name]
        )

        # Change the driver to force the network to be replaced
        ret = self.run_state(
            'docker_network.present',
            name=net.name,
            driver='macvlan',
            reconnect=reconnect)
        self.assertSaltTrueReturn(ret)
        ret = ret[next(iter(ret))]

        self.assertEqual(
            ret['changes'],
            {
                'recreated': True,
                'reconnected' if reconnect else 'disconnected': [container_name],
                net.name: {
                    'Driver': {
                        'old': 'bridge',
                        'new': 'macvlan',
                    }
                }
            }
        )
        self.assertEqual(
            ret['comment'],
            'Network \'{0}\' was replaced with updated config'.format(net.name)
        )

    @container_name
    @with_network(create=False)
    def test_present_with_reconnect(self, net, container_name):
        '''
        Test reconnecting with containers not passed to state
        '''
        self._test_present_reconnect(net, container_name, reconnect=True)

    @container_name
    @with_network(create=False)
    def test_present_with_no_reconnect(self, net, container_name):
        '''
        Test reconnecting with containers not passed to state
        '''
        self._test_present_reconnect(net, container_name, reconnect=False)

    @with_network()
    def test_present_internal(self, net):
        self.assertSaltTrueReturn(
            self.run_state(
                'docker_network.present',
                name=net.name,
                internal=True,
            )
        )
        net_info = self.run_function('docker.inspect_network', [net.name])
        self.assertIs(net_info['Internal'], True)

    @with_network()
    def test_present_labels(self, net):
        # Test a mix of different ways of specifying labels
        self.assertSaltTrueReturn(
            self.run_state(
                'docker_network.present',
                name=net.name,
                labels=[
                    'foo',
                    'bar=baz',
                    {'hello': 'world'},
                ],
            )
        )
        net_info = self.run_function('docker.inspect_network', [net.name])
        self.assertEqual(
            net_info['Labels'],
            {'foo': '',
             'bar': 'baz',
             'hello': 'world'},
        )

    @with_network(subnet='fe3f:2180:26:1::/123')
    @with_network(subnet='10.247.197.96/27')
    @skipIf(not IPV6_ENABLED, 'IPv6 not enabled')
    def test_present_enable_ipv6(self, net1, net2):
        self.assertSaltTrueReturn(
            self.run_state(
                'docker_network.present',
                name=net1.name,
                enable_ipv6=True,
                ipam_pools=[
                    {'subnet': net1.subnet},
                    {'subnet': net2.subnet},
                ],
            )
        )
        net_info = self.run_function('docker.inspect_network', [net1.name])
        self.assertIs(net_info['EnableIPv6'], True)

    @requires_system_grains
    @with_network()
    def test_present_attachable(self, net, grains):
        if grains['os_family'] == 'RedHat' \
                and grains.get('osmajorrelease', 0) <= 7:
            self.skipTest('Cannot reliably manage attachable on RHEL <= 7')

        self.assertSaltTrueReturn(
            self.run_state(
                'docker_network.present',
                name=net.name,
                attachable=True,
            )
        )
        net_info = self.run_function('docker.inspect_network', [net.name])
        self.assertIs(net_info['Attachable'], True)

    @skipIf(True, 'Skip until we can set up docker swarm testing')
    @with_network()
    def test_present_scope(self, net):
        self.assertSaltTrueReturn(
            self.run_state(
                'docker_network.present',
                name=net.name,
                scope='global',
            )
        )
        net_info = self.run_function('docker.inspect_network', [net.name])
        self.assertIs(net_info['Scope'], 'global')

    @skipIf(True, 'Skip until we can set up docker swarm testing')
    @with_network()
    def test_present_ingress(self, net):
        self.assertSaltTrueReturn(
            self.run_state(
                'docker_network.present',
                name=net.name,
                ingress=True,
            )
        )
        net_info = self.run_function('docker.inspect_network', [net.name])
        self.assertIs(net_info['Ingress'], True)

    @with_network(subnet='10.247.197.128/27')
    @with_network(subnet='10.247.197.96/27')
    def test_present_with_custom_ipv4(self, net1, net2):
        # First run will test passing the IPAM arguments individually
        self.assertSaltTrueReturn(
            self.run_state(
                'docker_network.present',
                name=net1.name,
                subnet=net1.subnet,
                gateway=net1.gateway,
            )
        )
        # Second run will pass them in the ipam_pools argument
        ret = self.run_state(
            'docker_network.present',
            name=net1.name,  # We want to keep the same network name
            ipam_pools=[
                {'subnet': net2.subnet,
                 'gateway': net2.gateway},
            ],
        )
        self.assertSaltTrueReturn(ret)
        ret = ret[next(iter(ret))]

        # Docker requires there to be IPv4, even when only an IPv6 subnet was
        # provided. So, there will be both an IPv4 and IPv6 pool in the
        # configuration.
        expected = {
            'recreated': True,
            net1.name: {
                'IPAM': {
                    'Config': {
                        'old': [
                            {'Subnet': net1.subnet,
                             'Gateway': net1.gateway},
                        ],
                        'new': [
                            {'Subnet': net2.subnet,
                             'Gateway': net2.gateway},
                        ],
                    }
                }
            }
        }
        self.assertEqual(ret['changes'], expected)
        self.assertEqual(
            ret['comment'],
            'Network \'{0}\' was replaced with updated config'.format(
                net1.name
            )
        )

    @with_network(subnet='fe3f:2180:26:1::20/123')
    @with_network(subnet='fe3f:2180:26:1::/123')
    @with_network(subnet='10.247.197.96/27')
    @skipIf(not IPV6_ENABLED, 'IPv6 not enabled')
    def test_present_with_custom_ipv6(self, ipv4_net, ipv6_net1, ipv6_net2):
        self.assertSaltTrueReturn(
            self.run_state(
                'docker_network.present',
                name=ipv4_net.name,
                enable_ipv6=True,
                ipam_pools=[
                    {'subnet': ipv4_net.subnet,
                     'gateway': ipv4_net.gateway},
                    {'subnet': ipv6_net1.subnet,
                     'gateway': ipv6_net1.gateway}
                ],
            )
        )

        ret = self.run_state(
            'docker_network.present',
            name=ipv4_net.name,  # We want to keep the same network name
            enable_ipv6=True,
            ipam_pools=[
                {'subnet': ipv4_net.subnet,
                 'gateway': ipv4_net.gateway},
                {'subnet': ipv6_net2.subnet,
                 'gateway': ipv6_net2.gateway}
            ],
        )
        self.assertSaltTrueReturn(ret)
        ret = ret[next(iter(ret))]

        # Docker requires there to be IPv4, even when only an IPv6 subnet was
        # provided. So, there will be both an IPv4 and IPv6 pool in the
        # configuration.
        expected = {
            'recreated': True,
            ipv4_net.name: {
                'IPAM': {
                    'Config': {
                        'old': [
                            {'Subnet': ipv4_net.subnet,
                             'Gateway': ipv4_net.gateway},
                            {'Subnet': ipv6_net1.subnet,
                             'Gateway': ipv6_net1.gateway}
                        ],
                        'new': [
                            {'Subnet': ipv4_net.subnet,
                             'Gateway': ipv4_net.gateway},
                            {'Subnet': ipv6_net2.subnet,
                             'Gateway': ipv6_net2.gateway}
                        ],
                    }
                }
            }
        }
        self.assertEqual(ret['changes'], expected)
        self.assertEqual(
            ret['comment'],
            'Network \'{0}\' was replaced with updated config'.format(
                ipv4_net.name
            )
        )



# coding: utf-8

# Python libs
from __future__ import absolute_import
from collections import namedtuple

# Salt testing libs
from tests.support.unit import skipIf, TestCase
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, patch, MagicMock
from tests.support.mixins import LoaderModuleMockMixin

# Salt libs
import salt.beacons.memusage as memusage

STUB_MEMORY_USAGE = namedtuple('vmem', 'total available percent used free active inactive buffers cached shared')(
                               15722012672, 9329594368, 40.7, 5137018880,
                               4678086656, 6991405056, 2078953472,
                               1156378624, 4750528512, 898908160)


@skipIf(NO_MOCK, NO_MOCK_REASON)
class MemUsageBeaconTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test case for salt.beacons.memusage
    '''

    def setup_loader_modules(self):
        return {}

    def test_non_list_config(self):
        config = {}

        ret = memusage.validate(config)

        self.assertEqual(ret, (False, 'Configuration for memusage beacon must'
                                      ' be a list.'))

    def test_empty_config(self):
        config = [{}]

        ret = memusage.validate(config)

        self.assertEqual(ret, (False, 'Configuration for memusage beacon '
                                     'requires percent.'))

    def test_memusage_match(self):
        with patch('psutil.virtual_memory',
                   MagicMock(return_value=STUB_MEMORY_USAGE)):

            config = [{'percent': '40%'}, {'interval': 30}]

            ret = memusage.validate(config)

            self.assertEqual(ret, (True, 'Valid beacon configuration'))

            ret = memusage.beacon(config)
            self.assertEqual(ret, [{'memusage': 40.7}])

    def test_memusage_nomatch(self):
        with patch('psutil.virtual_memory',
                   MagicMock(return_value=STUB_MEMORY_USAGE)):

            config = [{'percent': '70%'}]

            ret = memusage.validate(config)

            self.assertEqual(ret, (True, 'Valid beacon configuration'))

            ret = memusage.beacon(config)
            self.assertNotEqual(ret, [{'memusage': 50}])



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing libs
from tests.support.unit import TestCase

# Import Salt libs
import salt.utils.dictdiffer as dictdiffer


NONE = dictdiffer.RecursiveDictDiffer.NONE_VALUE


class RecursiveDictDifferTestCase(TestCase):

    def setUp(self):
        old_dict = {'a': {'b': 1, 'c': 2, 'e': 'old_value',
                          'f': 'old_key'},
                    'j': 'value'}
        new_dict = {'a': {'b': 1, 'c': 4, 'e': 'new_value',
                          'g': 'new_key'},
                    'h': 'new_key', 'i': None,
                    'j': 'value'}
        self.recursive_diff = \
                dictdiffer.recursive_diff(old_dict, new_dict,
                                          ignore_missing_keys=False)
        self.recursive_diff_ign = dictdiffer.recursive_diff(old_dict, new_dict)

    def tearDown(self):
        for attrname in ('recursive_diff', 'recursive_diff_missing_keys'):
            try:
                delattr(self, attrname)
            except AttributeError:
                continue

    def test_added(self):
        self.assertEqual(self.recursive_diff.added(), ['a.g', 'h', 'i'])

    def test_removed(self):
        self.assertEqual(self.recursive_diff.removed(), ['a.f'])

    def test_changed_with_ignore_unset_values(self):
        self.recursive_diff.ignore_unset_values = True
        self.assertEqual(self.recursive_diff.changed(),
                         ['a.c', 'a.e'])

    def test_changed_without_ignore_unset_values(self):
        self.recursive_diff.ignore_unset_values = False
        self.assertEqual(self.recursive_diff.changed(),
                         ['a.c', 'a.e', 'a.f', 'a.g', 'h', 'i'])

    def test_unchanged(self):
        self.assertEqual(self.recursive_diff.unchanged(),
                         ['a.b', 'j'])

    def test_diffs(self):
        self.assertDictEqual(self.recursive_diff.diffs,
                             {'a': {'c': {'old': 2, 'new': 4},
                                    'e': {'old': 'old_value',
                                          'new': 'new_value'},
                                    'f': {'old': 'old_key', 'new': NONE},
                                    'g': {'old': NONE, 'new': 'new_key'}},
                              'h': {'old': NONE, 'new': 'new_key'},
                              'i': {'old': NONE, 'new': None}})
        self.assertDictEqual(self.recursive_diff_ign.diffs,
                             {'a': {'c': {'old': 2, 'new': 4},
                                    'e': {'old': 'old_value',
                                          'new': 'new_value'},
                                    'g': {'old': NONE, 'new': 'new_key'}},
                              'h': {'old': NONE, 'new': 'new_key'},
                              'i': {'old': NONE, 'new': None}})

    def test_new_values(self):
        self.assertDictEqual(self.recursive_diff.new_values,
                             {'a': {'c': 4, 'e': 'new_value',
                                    'f': NONE, 'g': 'new_key'},
                              'h': 'new_key', 'i': None})

    def test_old_values(self):
        self.assertDictEqual(self.recursive_diff.old_values,
                             {'a': {'c': 2, 'e': 'old_value',
                                    'f': 'old_key', 'g': NONE},
                              'h': NONE, 'i': NONE})

    def test_changes_str(self):
        self.assertEqual(self.recursive_diff.changes_str,
                         'a:\n'
                         '  c from 2 to 4\n'
                         '  e from \'old_value\' to \'new_value\'\n'
                         '  f from \'old_key\' to nothing\n'
                         '  g from nothing to \'new_key\'\n'
                         'h from nothing to \'new_key\'\n'
                         'i from nothing to None')



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.unit import skipIf

# Import Salt Libs
import salt.utils.path
import salt.utils.platform

URL = 'repo.saltstack.com'


class NetworkTest(ModuleCase):
    '''
    Validate network module
    '''
    def test_network_ping(self):
        '''
        network.ping
        '''
        ret = self.run_function('network.ping', [URL])
        exp_out = ['ping', URL, 'ttl', 'time']
        for out in exp_out:
            self.assertIn(out, ret.lower())

    @skipIf(salt.utils.platform.is_darwin(), 'not supported on macosx')
    def test_network_netstat(self):
        '''
        network.netstat
        '''
        ret = self.run_function('network.netstat')
        exp_out = ['proto', 'local-address']
        for val in ret:
            for out in exp_out:
                self.assertIn(out, val)

    def test_network_traceroute(self):
        '''
        network.traceroute
        '''
        if not salt.utils.path.which('traceroute') and not salt.utils.platform.is_windows():
            self.skipTest('traceroute not installed')
        ret = self.run_function('network.traceroute', [URL])
        exp_out = ['hostname', 'ip']
        for out in exp_out:
            self.assertIn(out, exp_out)

    @skipIf(not salt.utils.platform.is_windows(), 'windows only test')
    def test_network_nslookup(self):
        '''
        network.nslookup
        '''
        ret = self.run_function('network.nslookup', [URL])
        exp_out = ['Server', 'Address']
        for out in exp_out:
            self.assertIn(out, exp_out)



# -*- coding: utf-8 -*-
'''
    :synopsis: Unit Tests for Windows SNMP Module 'module.win_snmp'
    :platform: Windows
    :maturity: develop
    .. versionadded:: 2017.7.0
'''

# Import Python Libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Libs
import salt.modules.win_snmp as win_snmp
from salt.exceptions import CommandExecutionError

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON,
)

COMMUNITY_NAMES = {'TestCommunity': 'Read Create'}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class WinSnmpTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.win_snmp
    '''
    def setup_loader_modules(self):
        return {win_snmp: {}}

    def test_get_agent_service_types(self):
        '''
        Test - Get the sysServices types that can be configured.
        '''
        with patch.dict(win_snmp.__salt__):
            self.assertIsInstance(win_snmp.get_agent_service_types(), list)

    def test_get_permission_types(self):
        '''
        Test - Get the permission types that can be configured for communities.
        '''
        with patch.dict(win_snmp.__salt__):
            self.assertIsInstance(win_snmp.get_permission_types(), list)

    def test_get_auth_traps_enabled(self):
        '''
        Test - Determine whether the host is configured to send authentication traps.
        '''
        mock_value = MagicMock(return_value={'vdata': 1})
        with patch.dict(win_snmp.__salt__, {'reg.read_value': mock_value}):
            self.assertTrue(win_snmp.get_auth_traps_enabled())

    def test_set_auth_traps_enabled(self):
        '''
        Test - Manage the sending of authentication traps.
        '''
        mock_value = MagicMock(return_value=True)
        kwargs = {'status': True}
        with patch.dict(win_snmp.__salt__, {'reg.set_value': mock_value}), \
                patch('salt.modules.win_snmp.get_auth_traps_enabled',
                      MagicMock(return_value=True)):
            self.assertTrue(win_snmp.set_auth_traps_enabled(**kwargs))

    def test_get_community_names(self):
        '''
        Test - Get the current accepted SNMP community names and their permissions.
        '''
        mock_ret = MagicMock(return_value=[{'vdata': 16,
                                            'vname': 'TestCommunity'}])
        mock_false = MagicMock(return_value=False)
        with patch.dict(win_snmp.__salt__, {'reg.list_values': mock_ret,
                                            'reg.key_exists': mock_false}):
            self.assertEqual(win_snmp.get_community_names(),
                             COMMUNITY_NAMES)

    def test_get_community_names_gpo(self):
        '''
        Test - Get the current accepted SNMP community names and their permissions.
        '''
        mock_ret = MagicMock(return_value=[{'vdata': 'TestCommunity',
                                            'vname': 1}])
        mock_false = MagicMock(return_value=True)
        with patch.dict(win_snmp.__salt__, {'reg.list_values': mock_ret,
                                            'reg.key_exists': mock_false}):
            self.assertEqual(win_snmp.get_community_names(),
                             {'TestCommunity': 'Managed by GPO'})

    def test_set_community_names(self):
        '''
        Test - Manage the SNMP accepted community names and their permissions.
        '''
        mock_true = MagicMock(return_value=True)
        kwargs = {'communities': COMMUNITY_NAMES}
        mock_false = MagicMock(return_value=False)
        with patch.dict(win_snmp.__salt__, {'reg.set_value': mock_true,
                                            'reg.key_exists': mock_false}), \
                patch('salt.modules.win_snmp.get_community_names',
                      MagicMock(return_value=COMMUNITY_NAMES)):
            self.assertTrue(win_snmp.set_community_names(**kwargs))

    def test_set_community_names_gpo(self):
        '''
        Test - Manage the SNMP accepted community names and their permissions.
        '''
        mock_true = MagicMock(return_value=True)
        kwargs = {'communities': COMMUNITY_NAMES}
        with patch.dict(win_snmp.__salt__, {'reg.set_value': mock_true,
                                            'reg.key_exists': mock_true}), \
             patch('salt.modules.win_snmp.get_community_names',
                   MagicMock(return_value=COMMUNITY_NAMES)):
            self.assertRaises(CommandExecutionError, win_snmp.set_community_names, **kwargs)



# -*- coding: utf-8 -*-
'''
Tests for salt.states.zpool

:codeauthor:    Jorge Schrauwen <sjorge@blackdot.be>
:maintainer:    Jorge Schrauwen <sjorge@blackdot.be>
:maturity:      new
:depends:       salt.utils.zfs, salt.modules.zpool
:platform:      illumos,freebsd,linux
'''
# Import Python libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch)

# Import test data from salt.utils.zfs test
from tests.unit.utils.test_zfs import utils_patch

# Import Salt Execution module to test
import salt.utils.zfs
import salt.states.zpool as zpool

# Import Salt Utils
import salt.loader
from salt.utils.odict import OrderedDict


@skipIf(NO_MOCK, NO_MOCK_REASON)
class ZpoolTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.states.zpool
    '''
    def setup_loader_modules(self):
        self.opts = opts = salt.config.DEFAULT_MINION_OPTS
        utils = salt.loader.utils(opts, whitelist=['zfs'])
        zpool_obj = {
            zpool: {
                '__opts__': opts,
                '__grains__': {'kernel': 'SunOS'},
                '__utils__': utils,
            }
        }

        return zpool_obj

    def test_absent_without_pool(self):
        '''
        Test zpool absent without a pool
        '''
        ret = {'name': 'myzpool',
               'result': True,
               'comment': 'storage pool myzpool is absent',
               'changes': {}}

        mock_exists = MagicMock(return_value=False)
        with patch.dict(zpool.__salt__, {'zpool.exists': mock_exists}), \
             patch.dict(zpool.__utils__, utils_patch):
            self.assertEqual(zpool.absent('myzpool'), ret)

    def test_absent_destroy_pool(self):
        '''
        Test zpool absent destroying pool
        '''
        ret = {
            'name': 'myzpool',
            'result': True,
            'comment': 'storage pool myzpool was destroyed',
            'changes': {'myzpool': 'destroyed'},
        }

        mock_exists = MagicMock(return_value=True)
        mock_destroy = MagicMock(return_value=OrderedDict([
            ('destroyed', True),
        ]))
        with patch.dict(zpool.__salt__, {'zpool.exists': mock_exists}), \
             patch.dict(zpool.__salt__, {'zpool.destroy': mock_destroy}), \
             patch.dict(zpool.__utils__, utils_patch):
            self.assertEqual(zpool.absent('myzpool'), ret)

    def test_absent_exporty_pool(self):
        '''
        Test zpool absent exporting pool
        '''
        ret = {
            'name': 'myzpool',
            'result': True,
            'comment': 'storage pool myzpool was exported',
            'changes': {'myzpool': 'exported'},
        }

        mock_exists = MagicMock(return_value=True)
        mock_destroy = MagicMock(return_value=OrderedDict([
            ('exported', True),
        ]))
        with patch.dict(zpool.__salt__, {'zpool.exists': mock_exists}), \
             patch.dict(zpool.__salt__, {'zpool.export': mock_destroy}), \
             patch.dict(zpool.__utils__, utils_patch):
            self.assertEqual(zpool.absent('myzpool', export=True), ret)

    def test_absent_busy(self):
        '''
        Test zpool absent on a busy pool
        '''
        ret = {
            'name': 'myzpool',
            'result': False,
            'comment': "\n".join([
                "cannot unmount '/myzpool': Device busy",
                "cannot export 'myzpool': pool is busy",
            ]),
            'changes': {},
        }

        mock_exists = MagicMock(return_value=True)
        mock_destroy = MagicMock(return_value=OrderedDict([
            ('exported', False),
            ('error', "\n".join([
                "cannot unmount '/myzpool': Device busy",
                "cannot export 'myzpool': pool is busy",
            ])),
        ]))
        with patch.dict(zpool.__salt__, {'zpool.exists': mock_exists}), \
             patch.dict(zpool.__salt__, {'zpool.export': mock_destroy}), \
             patch.dict(zpool.__utils__, utils_patch):
            self.assertEqual(zpool.absent('myzpool', export=True), ret)

    def test_present_import_success(self):
        '''
        Test zpool present with import allowed and unimported pool
        '''
        ret = {'name': 'myzpool',
               'result': True,
               'comment': 'storage pool myzpool was imported',
               'changes': {'myzpool': 'imported'}}

        config = {
            'import': True,
        }

        mock_exists = MagicMock(return_value=False)
        mock_import = MagicMock(return_value=OrderedDict([
            ('imported', True),
        ]))
        with patch.dict(zpool.__salt__, {'zpool.exists': mock_exists}), \
             patch.dict(zpool.__salt__, {'zpool.import': mock_import}), \
             patch.dict(zpool.__utils__, utils_patch):
            self.assertEqual(zpool.present('myzpool', config=config), ret)

    def test_present_import_fail(self):
        '''
        Test zpool present with import allowed and no unimported pool or layout
        '''
        ret = {'name': 'myzpool',
               'result': False,
               'comment': 'storage pool myzpool was not imported, no (valid) layout specified for creation',
               'changes': {}}

        config = {
            'import': True,
        }

        mock_exists = MagicMock(return_value=False)
        mock_import = MagicMock(return_value=OrderedDict([
            ('imported', False),
        ]))
        with patch.dict(zpool.__salt__, {'zpool.exists': mock_exists}), \
             patch.dict(zpool.__salt__, {'zpool.import': mock_import}), \
             patch.dict(zpool.__utils__, utils_patch):
            self.assertEqual(zpool.present('myzpool', config=config), ret)

    def test_present_create_success(self):
        '''
        Test zpool present with non existing pool
        '''
        ret = {'name': 'myzpool',
               'result': True,
               'comment': 'storage pool myzpool was created',
               'changes': {'myzpool': 'created'}}

        config = {
            'import': False,
        }
        layout = [
            OrderedDict([('mirror', ['disk0', 'disk1'])]),
            OrderedDict([('mirror', ['disk2', 'disk3'])]),
        ]
        properties = {
            'autoexpand': True,
        }
        filesystem_properties = {
            'quota': '5G',
        }

        mock_exists = MagicMock(return_value=False)
        mock_create = MagicMock(return_value=OrderedDict([
            ('created', True),
            ('vdevs', OrderedDict([
                ('mirror-0', ['/dev/dsk/disk0', '/dev/dsk/disk1']),
                ('mirror-1', ['/dev/dsk/disk2', '/dev/dsk/disk3']),
            ])),
        ]))
        with patch.dict(zpool.__salt__, {'zpool.exists': mock_exists}), \
             patch.dict(zpool.__salt__, {'zpool.create': mock_create}), \
             patch.dict(zpool.__utils__, utils_patch):
            self.assertEqual(
                zpool.present(
                    'myzpool',
                    config=config,
                    layout=layout,
                    properties=properties,
                    filesystem_properties=filesystem_properties,
                ),
                ret,
            )

    def test_present_create_fail(self):
        '''
        Test zpool present with non existing pool (without a layout)
        '''
        ret = {'name': 'myzpool',
               'result': False,
               'comment': 'storage pool myzpool was not imported, no (valid) layout specified for creation',
               'changes': {}}

        config = {
            'import': False,
        }

        mock_exists = MagicMock(return_value=False)
        with patch.dict(zpool.__salt__, {'zpool.exists': mock_exists}), \
             patch.dict(zpool.__utils__, utils_patch):
            self.assertEqual(zpool.present('myzpool', config=config), ret)

    def test_present_create_passthrough_fail(self):
        '''
        Test zpool present with non existing pool (without a layout)
        '''
        ret = {'name': 'myzpool',
               'result': False,
               'comment': "\n".join([
                    "invalid vdev specification",
                    "use 'force=True' to override the following errors:",
                    "/data/salt/vdisk0 is part of exported pool 'zsalt'",
                    "/data/salt/vdisk1 is part of exported pool 'zsalt'",
                ]),
               'changes': {}}

        config = {
            'force': False,
            'import': False,
        }
        layout = [
            OrderedDict([('mirror', ['disk0', 'disk1'])]),
            OrderedDict([('mirror', ['disk2', 'disk3'])]),
        ]
        properties = {
            'autoexpand': True,
        }
        filesystem_properties = {
            'quota': '5G',
        }

        mock_exists = MagicMock(return_value=False)
        mock_create = MagicMock(return_value=OrderedDict([
            ('created', False),
            ('error', "\n".join([
                "invalid vdev specification",
                "use 'force=True' to override the following errors:",
                "/data/salt/vdisk0 is part of exported pool 'zsalt'",
                "/data/salt/vdisk1 is part of exported pool 'zsalt'",
            ])),
        ]))
        with patch.dict(zpool.__salt__, {'zpool.exists': mock_exists}), \
             patch.dict(zpool.__salt__, {'zpool.create': mock_create}), \
             patch.dict(zpool.__utils__, utils_patch):
            self.assertEqual(
                zpool.present(
                    'myzpool',
                    config=config,
                    layout=layout,
                    properties=properties,
                    filesystem_properties=filesystem_properties,
                ),
                ret,
            )

    def test_present_update_success(self):
        '''
        Test zpool present with an existing pool that needs an update
        '''
        ret = {'name': 'myzpool',
               'result': True,
               'comment': 'properties updated',
               'changes': {'myzpool': {'autoexpand': False}}}

        config = {
            'import': False,
        }
        layout = [
            OrderedDict([('mirror', ['disk0', 'disk1'])]),
            OrderedDict([('mirror', ['disk2', 'disk3'])]),
        ]
        properties = {
            'autoexpand': False,
        }

        mock_exists = MagicMock(return_value=True)
        mock_get = MagicMock(return_value=OrderedDict([
            ('comment', 'salt managed pool'),
            ('freeing', 0),
            ('listsnapshots', False),
            ('leaked', 0),
            ('feature@obsolete_counts', 'enabled'),
            ('feature@sha512', 'enabled'),
            ('delegation', True),
            ('dedupditto', '0'),
            ('dedupratio', '1.00x'),
            ('autoexpand', True),
            ('feature@bookmarks', 'enabled'),
            ('allocated', 115712),
            ('guid', 1591906802560842214),
            ('feature@large_blocks', 'enabled'),
            ('size', 2113929216),
            ('feature@enabled_txg', 'active'),
            ('feature@hole_birth', 'active'),
            ('capacity', 0),
            ('feature@multi_vdev_crash_dump', 'enabled'),
            ('feature@extensible_dataset', 'enabled'),
            ('cachefile', '-'),
            ('bootfs', '-'),
            ('autoreplace', True),
            ('readonly', False),
            ('version', '-'),
            ('health', 'ONLINE'),
            ('expandsize', '-'),
            ('feature@embedded_data', 'active'),
            ('feature@lz4_compress', 'active'),
            ('feature@async_destroy', 'enabled'),
            ('feature@skein', 'enabled'),
            ('feature@empty_bpobj', 'enabled'),
            ('feature@spacemap_histogram', 'active'),
            ('bootsize', '-'),
            ('free', 2113813504),
            ('feature@device_removal', 'enabled'),
            ('failmode', 'wait'),
            ('feature@filesystem_limits', 'enabled'),
            ('feature@edonr', 'enabled'),
            ('altroot', '-'),
            ('fragmentation', '0%'),
        ]))
        mock_set = MagicMock(return_value=OrderedDict([
            ('set', True),
        ]))
        with patch.dict(zpool.__salt__, {'zpool.exists': mock_exists}), \
             patch.dict(zpool.__salt__, {'zpool.get': mock_get}), \
             patch.dict(zpool.__salt__, {'zpool.set': mock_set}), \
             patch.dict(zpool.__utils__, utils_patch):
            self.assertEqual(
                zpool.present(
                    'myzpool',
                    config=config,
                    layout=layout,
                    properties=properties,
                ),
                ret,
            )

    def test_present_update_nochange_success(self):
        '''
        Test zpool present with non existing pool
        '''
        ret = {'name': 'myzpool',
               'result': True,
               'comment': 'no update needed',
               'changes': {}}

        config = {
            'import': False,
        }
        layout = [
            OrderedDict([('mirror', ['disk0', 'disk1'])]),
            OrderedDict([('mirror', ['disk2', 'disk3'])]),
        ]
        properties = {
            'autoexpand': True,
        }

        mock_exists = MagicMock(return_value=True)
        mock_get = MagicMock(return_value=OrderedDict([
            ('comment', 'salt managed pool'),
            ('freeing', 0),
            ('listsnapshots', False),
            ('leaked', 0),
            ('feature@obsolete_counts', 'enabled'),
            ('feature@sha512', 'enabled'),
            ('delegation', True),
            ('dedupditto', '0'),
            ('dedupratio', '1.00x'),
            ('autoexpand', True),
            ('feature@bookmarks', 'enabled'),
            ('allocated', 115712),
            ('guid', 1591906802560842214),
            ('feature@large_blocks', 'enabled'),
            ('size', 2113929216),
            ('feature@enabled_txg', 'active'),
            ('feature@hole_birth', 'active'),
            ('capacity', 0),
            ('feature@multi_vdev_crash_dump', 'enabled'),
            ('feature@extensible_dataset', 'enabled'),
            ('cachefile', '-'),
            ('bootfs', '-'),
            ('autoreplace', True),
            ('readonly', False),
            ('version', '-'),
            ('health', 'ONLINE'),
            ('expandsize', '-'),
            ('feature@embedded_data', 'active'),
            ('feature@lz4_compress', 'active'),
            ('feature@async_destroy', 'enabled'),
            ('feature@skein', 'enabled'),
            ('feature@empty_bpobj', 'enabled'),
            ('feature@spacemap_histogram', 'active'),
            ('bootsize', '-'),
            ('free', 2113813504),
            ('feature@device_removal', 'enabled'),
            ('failmode', 'wait'),
            ('feature@filesystem_limits', 'enabled'),
            ('feature@edonr', 'enabled'),
            ('altroot', '-'),
            ('fragmentation', '0%'),
        ]))
        with patch.dict(zpool.__salt__, {'zpool.exists': mock_exists}), \
             patch.dict(zpool.__salt__, {'zpool.get': mock_get}), \
             patch.dict(zpool.__utils__, utils_patch):
            self.assertEqual(
                zpool.present(
                    'myzpool',
                    config=config,
                    layout=layout,
                    properties=properties,
                ),
                ret,
            )



# -*- coding: utf-8 -*-
'''
    :codeauthor: Rupesh Tare <rupesht@saltstack.com>
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import sys

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.modules.genesis as genesis


@skipIf(NO_MOCK, NO_MOCK_REASON)
class GenesisTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.genesis
    '''
    def setup_loader_modules(self):
        return {genesis: {}}

    def test_bootstrap(self):
        '''
        Test for Create an image for a specific platform.
        '''
        # Changed in 3.7.0 pformat no longer includes the comma
        if sys.version_info >= (3, 7):
            exception_string = 'Exception({0})'.format(repr('foo'))
        else:
            exception_string = 'Exception({0},)'.format(repr('foo'))
        mock = MagicMock(return_value=False)
        with patch.dict(genesis.__salt__, {'file.directory_exists': mock}):
            mock = MagicMock(side_effect=Exception('foo'))
            with patch.dict(genesis.__salt__, {'file.mkdir': mock}):
                self.assertEqual(genesis.bootstrap('platform', 'root'),
                                 {'Error': exception_string})

        with patch.object(genesis, '_bootstrap_yum', return_value='A'):
            with patch.dict(genesis.__salt__, {'mount.umount': MagicMock(),
                                               'file.rmdir': MagicMock(),
                                               'file.directory_exists': MagicMock()}):
                with patch.dict(genesis.__salt__, {'disk.blkid': MagicMock(return_value={})}):
                    self.assertEqual(genesis.bootstrap('rpm', 'root', 'dir'), None)

        common_parms = {'platform': 'deb',
                        'root': 'root',
                        'img_format': 'dir',
                        'arch': 'amd64',
                        'flavor': 'stable',
                        'static_qemu': 'qemu'}

        param_sets = [

            {'params': {},
             'cmd': ['debootstrap', '--foreign', '--arch', 'amd64',
                     'stable', 'root', 'http://ftp.debian.org/debian/']
             },

            {'params': {'pkgs': 'vim'},
             'cmd': ['debootstrap', '--foreign', '--arch', 'amd64',
                     '--include', 'vim',
                     'stable', 'root', 'http://ftp.debian.org/debian/']
             },

            {'params': {'pkgs': 'vim,emacs'},
             'cmd': ['debootstrap', '--foreign', '--arch', 'amd64',
                     '--include', 'vim,emacs',
                     'stable', 'root', 'http://ftp.debian.org/debian/']
             },

            {'params': {'pkgs': ['vim', 'emacs']},
             'cmd': ['debootstrap', '--foreign', '--arch', 'amd64',
                     '--include', 'vim,emacs',
                     'stable', 'root', 'http://ftp.debian.org/debian/']
             },

            {'params': {'pkgs': ['vim', 'emacs'], 'exclude_pkgs': ['vim', 'foo']},
             'cmd': ['debootstrap', '--foreign', '--arch', 'amd64',
                     '--include', 'vim,emacs', '--exclude', 'vim,foo',
                     'stable', 'root', 'http://ftp.debian.org/debian/']
             },

        ]

        for param_set in param_sets:

            with patch.dict(genesis.__salt__, {'mount.umount': MagicMock(),
                                               'file.rmdir': MagicMock(),
                                               'file.directory_exists': MagicMock(),
                                               'cmd.run': MagicMock(),
                                               'disk.blkid': MagicMock(return_value={})}):
                with patch('salt.modules.genesis.salt.utils.path.which', return_value=True):
                    with patch('salt.modules.genesis.salt.utils.validate.path.is_executable',
                               return_value=True):
                        param_set['params'].update(common_parms)
                        self.assertEqual(genesis.bootstrap(**param_set['params']), None)
                        genesis.__salt__['cmd.run'].assert_any_call(param_set['cmd'], python_shell=False)

        with patch.object(genesis, '_bootstrap_pacman', return_value='A') as pacman_patch:
            with patch.dict(genesis.__salt__, {'mount.umount': MagicMock(),
                                               'file.rmdir': MagicMock(),
                                               'file.directory_exists': MagicMock(),
                                               'disk.blkid': MagicMock(return_value={})}):
                genesis.bootstrap('pacman', 'root', 'dir')
                pacman_patch.assert_called_with('root', img_format='dir', exclude_pkgs=[], pkgs=[])

    def test_avail_platforms(self):
        '''
        Test for Return which platforms are available
        '''
        with patch('salt.utils.path.which', MagicMock(return_value=False)):
            self.assertFalse(genesis.avail_platforms()['deb'])

    def test_pack(self):
        '''
        Test for Pack up a directory structure, into a specific format
        '''
        with patch.object(genesis, '_tar', return_value='tar'):
            self.assertEqual(genesis.pack('name', 'root'), None)

    def test_unpack(self):
        '''
        Test for Unpack an image into a directory structure
        '''
        with patch.object(genesis, '_untar', return_value='untar'):
            self.assertEqual(genesis.unpack('name', 'root'), None)



# -*- coding: utf-8 -*-
'''
Integration tests for the saltutil module.
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import os
import time
import textwrap

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.helpers import flaky
from tests.support.paths import TMP_PILLAR_TREE
from tests.support.unit import skipIf

# Import Salt Libs
import salt.utils.files
import salt.utils.stringutils


class SaltUtilModuleTest(ModuleCase):
    '''
    Testcase for the saltutil execution module
    '''

    def setUp(self):
        self.run_function('saltutil.refresh_pillar')

    # Tests for the wheel function

    def test_wheel_just_function(self):
        '''
        Tests using the saltutil.wheel function when passing only a function.
        '''
        # Wait for the pillar refresh to kick in, so that grains are ready to go
        time.sleep(3)
        ret = self.run_function('saltutil.wheel', ['minions.connected'])
        self.assertIn('minion', ret['return'])
        self.assertIn('sub_minion', ret['return'])

    def test_wheel_with_arg(self):
        '''
        Tests using the saltutil.wheel function when passing a function and an arg.
        '''
        ret = self.run_function('saltutil.wheel', ['key.list', 'minion'])
        self.assertEqual(ret['return'], {})

    def test_wheel_no_arg_raise_error(self):
        '''
        Tests using the saltutil.wheel function when passing a function that requires
        an arg, but one isn't supplied.
        '''
        self.assertRaises(TypeError, 'saltutil.wheel', ['key.list'])

    def test_wheel_with_kwarg(self):
        '''
        Tests using the saltutil.wheel function when passing a function and a kwarg.
        This function just generates a key pair, but doesn't do anything with it. We
        just need this for testing purposes.
        '''
        ret = self.run_function('saltutil.wheel', ['key.gen'], keysize=1024)
        self.assertIn('pub', ret['return'])
        self.assertIn('priv', ret['return'])


class SaltUtilSyncModuleTest(ModuleCase):
    '''
    Testcase for the saltutil sync execution module
    '''

    def setUp(self):
        whitelist = {'modules': [], }
        self.run_function('saltutil.sync_all', extmod_whitelist=whitelist)

    def tearDown(self):
        self.run_function('saltutil.sync_all')

    def test_sync_all(self):
        '''
        Test syncing all ModuleCase
        '''
        expected_return = {'engines': [],
                           'clouds': [],
                           'grains': [],
                           'beacons': [],
                           'utils': [],
                           'returners': [],
                           'modules': ['modules.mantest',
                                       'modules.override_test',
                                       'modules.runtests_decorators',
                                       'modules.runtests_helpers',
                                       'modules.salttest'],
                           'renderers': [],
                           'log_handlers': [],
                           'states': [],
                           'sdb': [],
                           'proxymodules': [],
                           'output': [],
                           'thorium': []}
        ret = self.run_function('saltutil.sync_all')
        self.assertEqual(ret, expected_return)

    def test_sync_all_whitelist(self):
        '''
        Test syncing all ModuleCase with whitelist
        '''
        expected_return = {'engines': [],
                           'clouds': [],
                           'grains': [],
                           'beacons': [],
                           'utils': [],
                           'returners': [],
                           'modules': ['modules.salttest'],
                           'renderers': [],
                           'log_handlers': [],
                           'states': [],
                           'sdb': [],
                           'proxymodules': [],
                           'output': [],
                           'thorium': []}
        ret = self.run_function('saltutil.sync_all', extmod_whitelist={'modules': ['salttest']})
        self.assertEqual(ret, expected_return)

    def test_sync_all_blacklist(self):
        '''
        Test syncing all ModuleCase with blacklist
        '''
        expected_return = {'engines': [],
                           'clouds': [],
                           'grains': [],
                           'beacons': [],
                           'utils': [],
                           'returners': [],
                           'modules': ['modules.mantest',
                                       'modules.override_test',
                                       'modules.runtests_helpers',
                                       'modules.salttest'],
                           'renderers': [],
                           'log_handlers': [],
                           'states': [],
                           'sdb': [],
                           'proxymodules': [],
                           'output': [],
                           'thorium': []}
        ret = self.run_function('saltutil.sync_all', extmod_blacklist={'modules': ['runtests_decorators']})
        self.assertEqual(ret, expected_return)

    def test_sync_all_blacklist_and_whitelist(self):
        '''
        Test syncing all ModuleCase with whitelist and blacklist
        '''
        expected_return = {'engines': [],
                           'clouds': [],
                           'grains': [],
                           'beacons': [],
                           'utils': [],
                           'returners': [],
                           'modules': [],
                           'renderers': [],
                           'log_handlers': [],
                           'states': [],
                           'sdb': [],
                           'proxymodules': [],
                           'output': [],
                           'thorium': []}
        ret = self.run_function('saltutil.sync_all', extmod_whitelist={'modules': ['runtests_decorators']},
                                extmod_blacklist={'modules': ['runtests_decorators']})
        self.assertEqual(ret, expected_return)


@skipIf(True, 'Pillar refresh test is flaky. Skipping for now.')
class SaltUtilSyncPillarTest(ModuleCase):
    '''
    Testcase for the saltutil sync pillar module
    '''

    @flaky
    def test_pillar_refresh(self):
        '''
        test pillar refresh module
        '''
        pillar_key = 'itworked'

        pre_pillar = self.run_function('pillar.raw')
        self.assertNotIn(pillar_key, pre_pillar.get(pillar_key, 'didnotwork'))

        with salt.utils.files.fopen(os.path.join(TMP_PILLAR_TREE, 'add_pillar.sls'), 'w') as fp:
            fp.write(salt.utils.stringutils.to_str(
                '{0}: itworked'.format(pillar_key)
            ))

        with salt.utils.files.fopen(os.path.join(TMP_PILLAR_TREE, 'top.sls'), 'w') as fp:
            fp.write(textwrap.dedent('''\
                     base:
                       '*':
                         - add_pillar
                     '''))

        self.run_function('saltutil.refresh_pillar')

        pillar = False
        timeout = time.time() + 30
        while not pillar:
            post_pillar = self.run_function('pillar.raw')
            try:
                self.assertIn(pillar_key, post_pillar.get(pillar_key, 'didnotwork'))
                pillar = True
            except AssertionError:
                if time.time() > timeout:
                    self.assertIn(pillar_key, post_pillar.get(pillar_key, 'didnotwork'))
                continue

        post_pillar = self.run_function('pillar.raw')
        self.assertIn(pillar_key, post_pillar.get(pillar_key, 'didnotwork'))

    def tearDown(self):
        for filename in os.listdir(TMP_PILLAR_TREE):
            os.remove(os.path.join(TMP_PILLAR_TREE, filename))



# -*- coding: utf-8 -*-
'''
    :synopsis: Unit Tests for 'module.yumkernelpkg'
    :platform: Linux
    :maturity: develop
    .. versionadded:: 2018.3.0
'''
# pylint: disable=invalid-name,no-member

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

try:
    # Import Salt Testing Libs
    from tests.support.mixins import LoaderModuleMockMixin
    from tests.support.unit import TestCase, skipIf
    from tests.support.mock import MagicMock, patch, NO_MOCK, NO_MOCK_REASON

    # Import Salt Libs
    from tests.support.kernelpkg import KernelPkgTestCase
    import salt.modules.kernelpkg_linux_yum as kernelpkg
    import salt.modules.yumpkg as pkg
    from salt.exceptions import CommandExecutionError
    HAS_MODULES = True
except ImportError:
    HAS_MODULES = False


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_MODULES, 'Salt modules could not be loaded')
class YumKernelPkgTestCase(KernelPkgTestCase, TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.kernelpkg_linux_yum
    '''

    _kernelpkg = kernelpkg
    KERNEL_LIST = ['3.10.0-327.el7', '3.11.0-327.el7', '4.9.1-100.el7']
    LATEST = KERNEL_LIST[-1]
    OS_ARCH = 'x86_64'
    OS_NAME = 'RedHat'

    def setup_loader_modules(self):
        return {
            kernelpkg: {
                '__grains__': {
                    'os': self.OS_NAME,
                    'kernelrelease': '{0}.{1}'.format(self.KERNEL_LIST[0], self.OS_ARCH)
                },
                '__salt__': {
                    'pkg.normalize_name': pkg.normalize_name,
                    'pkg.upgrade': MagicMock(return_value={}),
                    'pkg.list_pkgs': MagicMock(return_value={}),
                    'pkg.version': MagicMock(return_value=self.KERNEL_LIST),
                    'system.reboot': MagicMock(return_value=None),
                    'config.get': MagicMock(return_value=True)
                }
            },
            pkg: {
                '__grains__': {
                    'osarch': self.OS_ARCH
                }
            }
        }

    def test_list_installed(self):
        '''
        Test - Return the latest installed kernel version
        '''
        mock = MagicMock(return_value=self.KERNEL_LIST)
        with patch.dict(self._kernelpkg.__salt__, {'pkg.version': mock}):
            self.assertListEqual(self._kernelpkg.list_installed(), self.KERNEL_LIST)

    def test_list_installed_none(self):
        '''
        Test - Return the latest installed kernel version
        '''
        mock = MagicMock(return_value=None)
        with patch.dict(self._kernelpkg.__salt__, {'pkg.version': mock}):
            self.assertListEqual(self._kernelpkg.list_installed(), [])

    def test_remove_success(self):
        '''
        Test - remove kernel package
        '''
        mock = MagicMock(return_value={'retcode': 0, 'stderr': []})
        with patch.dict(self._kernelpkg.__salt__, {'cmd.run_all': mock}):
            with patch.object(self._kernelpkg, 'active', return_value=self.KERNEL_LIST[-1]):
                with patch.object(self._kernelpkg, 'list_installed', return_value=self.KERNEL_LIST):
                    result = self._kernelpkg.remove(release=self.KERNEL_LIST[0])
                    self.assertIn('removed', result)
                    target = '{0}-{1}'.format(self._kernelpkg._package_name(), self.KERNEL_LIST[0])  # pylint: disable=protected-access
                    self.assertListEqual(result['removed'], [target])

    def test_remove_error(self):
        '''
        Test - remove kernel package
        '''
        mock = MagicMock(return_value={'retcode': -1, 'stderr': []})
        with patch.dict(self._kernelpkg.__salt__, {'cmd.run_all': mock}):
            with patch.object(self._kernelpkg, 'active', return_value=self.KERNEL_LIST[-1]):
                with patch.object(self._kernelpkg, 'list_installed', return_value=self.KERNEL_LIST):
                    self.assertRaises(CommandExecutionError, self._kernelpkg.remove, release=self.KERNEL_LIST[0])



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import
import copy
import datetime
import logging
import os
import random
import time

import dateutil.parser as dateutil_parser
import datetime

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.mixins import SaltReturnAssertsMixin

# Import Salt Testing Libs
from tests.support.mock import MagicMock, patch
from tests.support.unit import skipIf
import tests.integration as integration

# Import Salt libs
import salt.utils.schedule
import salt.utils.platform

from salt.modules.test import ping as ping

try:
    import croniter  # pylint: disable=W0611
    HAS_CRONITER = True
except ImportError:
    HAS_CRONITER = False

log = logging.getLogger(__name__)
ROOT_DIR = os.path.join(integration.TMP, 'schedule-unit-tests')
SOCK_DIR = os.path.join(ROOT_DIR, 'test-socks')

DEFAULT_CONFIG = salt.config.minion_config(None)
DEFAULT_CONFIG['conf_dir'] = ROOT_DIR
DEFAULT_CONFIG['root_dir'] = ROOT_DIR
DEFAULT_CONFIG['sock_dir'] = SOCK_DIR
DEFAULT_CONFIG['pki_dir'] = os.path.join(ROOT_DIR, 'pki')
DEFAULT_CONFIG['cachedir'] = os.path.join(ROOT_DIR, 'cache')


class SchedulerEvalTest(ModuleCase, SaltReturnAssertsMixin):
    '''
    Validate the pkg module
    '''
    def setUp(self):
        with patch('salt.utils.schedule.clean_proc_dir', MagicMock(return_value=None)):
            functions = {'test.ping': ping}
            self.schedule = salt.utils.schedule.Schedule(copy.deepcopy(DEFAULT_CONFIG), functions, returners={})
        self.schedule.opts['loop_interval'] = 1

    def tearDown(self):
        self.schedule.reset()

    def test_eval(self):
        '''
        verify that scheduled job runs
        '''
        job_name = 'test_eval'
        job = {
          'schedule': {
            job_name: {
              'function': 'test.ping',
              'when': '11/29/2017 4:00pm',
            }
          }
        }
        run_time2 = dateutil_parser.parse('11/29/2017 4:00pm')
        run_time1 = run_time2 - datetime.timedelta(seconds=1)

        # Add the job to the scheduler
        self.schedule.opts.update(job)

        # Evaluate 1 second before the run time
        self.schedule.eval(now=run_time1)
        ret = self.schedule.job_status(job_name)
        self.assertNotIn('_last_run', ret)

        # Evaluate 1 second at the run time
        self.schedule.eval(now=run_time2)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_last_run'], run_time2)

    def test_eval_multiple_whens(self):
        '''
        verify that scheduled job runs
        '''
        job_name = 'test_eval_multiple_whens'
        job = {
          'schedule': {
            job_name: {
              'function': 'test.ping',
              'when': [
                '11/29/2017 4:00pm',
                '11/29/2017 5:00pm',
                ]
            }
          }
        }
        if salt.utils.platform.is_darwin():
            job['schedule'][job_name]['dry_run'] = True

        run_time1 = dateutil_parser.parse('11/29/2017 4:00pm')
        run_time2 = dateutil_parser.parse('11/29/2017 5:00pm')

        # Add the job to the scheduler
        self.schedule.opts.update(job)

        # Evaluate run time1
        self.schedule.eval(now=run_time1)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_last_run'], run_time1)

        time.sleep(2)

        # Evaluate run time2
        self.schedule.eval(now=run_time2)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_last_run'], run_time2)

    def test_eval_loop_interval(self):
        '''
        verify that scheduled job runs
        '''
        job_name = 'test_eval_loop_interval'
        job = {
          'schedule': {
            job_name: {
              'function': 'test.ping',
              'when': '11/29/2017 4:00pm',
            }
          }
        }
        # 30 second loop interval
        LOOP_INTERVAL = random.randint(30, 59)
        self.schedule.opts['loop_interval'] = LOOP_INTERVAL

        run_time2 = dateutil_parser.parse('11/29/2017 4:00pm')

        # Add the job to the scheduler
        self.schedule.opts.update(job)

        # Evaluate 1 second at the run time
        self.schedule.eval(now=run_time2 + datetime.timedelta(seconds=LOOP_INTERVAL))

        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_last_run'], run_time2 + datetime.timedelta(seconds=LOOP_INTERVAL))

    def test_eval_multiple_whens_loop_interval(self):
        '''
        verify that scheduled job runs
        '''
        job_name = 'test_eval_multiple_whens_loop_interval'
        job = {
          'schedule': {
            job_name: {
              'function': 'test.ping',
              'when': [
                '11/29/2017 4:00pm',
                '11/29/2017 5:00pm',
                ],
            }
          }
        }
        if salt.utils.platform.is_darwin():
            job['schedule'][job_name]['dry_run'] = True

        # 30 second loop interval
        LOOP_INTERVAL = random.randint(30, 59)
        self.schedule.opts['loop_interval'] = LOOP_INTERVAL

        run_time1 = dateutil_parser.parse('11/29/2017 4:00pm') + datetime.timedelta(seconds=LOOP_INTERVAL)
        run_time2 = dateutil_parser.parse('11/29/2017 5:00pm') + datetime.timedelta(seconds=LOOP_INTERVAL)

        # Add the job to the scheduler
        self.schedule.opts.update(job)

        # Evaluate 1 second at the run time
        self.schedule.eval(now=run_time1)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_last_run'], run_time1)

        time.sleep(2)

        # Evaluate 1 second at the run time
        self.schedule.eval(now=run_time2)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_last_run'], run_time2)

    def test_eval_once(self):
        '''
        verify that scheduled job runs
        '''
        job_name = 'test_once'
        job = {
          'schedule': {
            job_name: {
              'function': 'test.ping',
              'once': '2017-12-13T13:00:00',
            }
          }
        }
        run_time = dateutil_parser.parse('12/13/2017 1:00pm')

        # Add the job to the scheduler
        self.schedule.opts.update(job)

        # Evaluate 1 second at the run time
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_last_run'], run_time)

    def test_eval_once_loop_interval(self):
        '''
        verify that scheduled job runs
        '''
        job_name = 'test_eval_once_loop_interval'
        job = {
          'schedule': {
            job_name: {
              'function': 'test.ping',
              'once': '2017-12-13T13:00:00',
            }
          }
        }
        # Randomn second loop interval
        LOOP_INTERVAL = random.randint(0, 59)
        self.schedule.opts['loop_interval'] = LOOP_INTERVAL

        # Run the job at the right plus LOOP_INTERVAL
        run_time = dateutil_parser.parse('12/13/2017 1:00pm') + datetime.timedelta(seconds=LOOP_INTERVAL)

        # Add the job to the scheduler
        self.schedule.opts.update(job)

        # Evaluate at the run time
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_last_run'], run_time)

    @skipIf(not HAS_CRONITER, 'Cannot find croniter python module')
    def test_eval_cron(self):
        '''
        verify that scheduled job runs
        '''
        job_name = 'test_eval_cron'
        job = {
          'schedule': {
            job_name: {
              'function': 'test.ping',
              'cron': '0 16 29 11 *',
            }
          }
        }

        # Add the job to the scheduler
        self.schedule.opts['schedule'] = {}
        self.schedule.opts.update(job)

        run_time = dateutil_parser.parse('11/29/2017 4:00pm')
        with patch('croniter.croniter.get_next', MagicMock(return_value=run_time)):
            self.schedule.eval(now=run_time)

        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_last_run'], run_time)

    @skipIf(not HAS_CRONITER, 'Cannot find croniter python module')
    def test_eval_cron_loop_interval(self):
        '''
        verify that scheduled job runs
        '''
        job_name = 'test_eval_cron_loop_interval'
        job = {
          'schedule': {
            job_name: {
              'function': 'test.ping',
              'cron': '0 16 29 11 *',
            }
          }
        }
        # Randomn second loop interval
        LOOP_INTERVAL = random.randint(0, 59)
        self.schedule.opts['loop_interval'] = LOOP_INTERVAL

        # Add the job to the scheduler
        self.schedule.opts.update(job)

        run_time = dateutil_parser.parse('11/29/2017 4:00pm')
        with patch('croniter.croniter.get_next', MagicMock(return_value=run_time)):
            self.schedule.eval(now=run_time)

        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_last_run'], run_time)

    def test_eval_until(self):
        '''
        verify that scheduled job is skipped once the current
        time reaches the specified until time
        '''
        job_name = 'test_eval_until'
        job = {
          'schedule': {
            job_name: {
              'function': 'test.ping',
              'hours': '1',
              'until': '11/29/2017 5:00pm',
            }
          }
        }

        if salt.utils.platform.is_darwin():
            job['schedule'][job_name]['dry_run'] = True

        # Add job to schedule
        self.schedule.opts.update(job)

        # eval at 2:00pm to prime, simulate minion start up.
        run_time = dateutil_parser.parse('11/29/2017 2:00pm')
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)

        # eval at 3:00pm, will run.
        run_time = dateutil_parser.parse('11/29/2017 3:00pm')
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_last_run'], run_time)

        time.sleep(2)

        # eval at 4:00pm, will run.
        run_time = dateutil_parser.parse('11/29/2017 4:00pm')
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_last_run'], run_time)

        time.sleep(2)

        # eval at 5:00pm, will not run
        run_time = dateutil_parser.parse('11/29/2017 5:00pm')
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_skip_reason'], 'until_passed')
        self.assertEqual(ret['_skipped_time'], run_time)

    def test_eval_after(self):
        '''
        verify that scheduled job is skipped until after the specified
        time has been reached.
        '''
        job_name = 'test_eval_after'
        job = {
          'schedule': {
            job_name: {
              'function': 'test.ping',
              'hours': '1',
              'after': '11/29/2017 5:00pm',
            }
          }
        }

        # Add job to schedule
        self.schedule.opts.update(job)

        # eval at 2:00pm to prime, simulate minion start up.
        run_time = dateutil_parser.parse('11/29/2017 2:00pm')
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)

        # eval at 3:00pm, will not run.
        run_time = dateutil_parser.parse('11/29/2017 3:00pm')
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_skip_reason'], 'after_not_passed')
        self.assertEqual(ret['_skipped_time'], run_time)

        # eval at 4:00pm, will not run.
        run_time = dateutil_parser.parse('11/29/2017 4:00pm')
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_skip_reason'], 'after_not_passed')
        self.assertEqual(ret['_skipped_time'], run_time)

        # eval at 5:00pm, will not run
        run_time = dateutil_parser.parse('11/29/2017 5:00pm')
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_skip_reason'], 'after_not_passed')
        self.assertEqual(ret['_skipped_time'], run_time)

        # eval at 6:00pm, will run
        run_time = dateutil_parser.parse('11/29/2017 6:00pm')
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_last_run'], run_time)

    def test_eval_run_on_start(self):
        '''
        verify that scheduled job is run when minion starts
        '''
        job = {
          'schedule': {
            'job1': {
              'function': 'test.ping',
              'hours': '1',
              'run_on_start': True
            }
          }
        }

        # Add job to schedule
        self.schedule.opts.update(job)

        # eval at 2:00pm, will run.
        run_time = dateutil_parser.parse('11/29/2017 2:00pm')
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status('job1')
        self.assertEqual(ret['_last_run'], run_time)

        time.sleep(2)

        # eval at 3:00pm, will run.
        run_time = dateutil_parser.parse('11/29/2017 3:00pm')
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status('job1')
        self.assertEqual(ret['_last_run'], run_time)

    def test_eval_enabled(self):
        '''
        verify that scheduled job runs
        when the enabled key is in place
        https://github.com/saltstack/salt/issues/47695
        '''
        job_name = 'test_eval_enabled'
        job = {
          'schedule': {
            'enabled': True,
            job_name: {
              'function': 'test.ping',
              'when': '11/29/2017 4:00pm',
            }
          }
        }
        run_time2 = dateutil_parser.parse('11/29/2017 4:00pm')
        run_time1 = run_time2 - datetime.timedelta(seconds=1)

        # Add the job to the scheduler
        self.schedule.opts.update(job)

        # Evaluate 1 second before the run time
        self.schedule.eval(now=run_time1)
        ret = self.schedule.job_status(job_name)
        self.assertNotIn('_last_run', ret)

        # Evaluate 1 second at the run time
        self.schedule.eval(now=run_time2)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_last_run'], run_time2)

    def test_eval_seconds(self):
        '''
        verify that scheduled job run mutiple times with seconds
        '''
        job_name = 'job_eval_seconds'
        job = {
          'schedule': {
            job_name: {
              'function': 'test.ping',
              'seconds': '30',
            }
          }
        }

        if salt.utils.platform.is_darwin():
            job['schedule'][job_name]['dry_run'] = True

        # Add job to schedule
        self.schedule.opts.update(job)

        # eval at 2:00pm to prime, simulate minion start up.
        run_time = dateutil_parser.parse('11/29/2017 2:00pm')
        next_run_time = run_time + datetime.timedelta(seconds=30)
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_next_fire_time'], next_run_time)

        # eval at 2:00:01pm, will not run.
        run_time = dateutil_parser.parse('11/29/2017 2:00:01pm')
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertNotIn('_last_run', ret)
        self.assertEqual(ret['_next_fire_time'], next_run_time)

        # eval at 2:00:30pm, will run.
        run_time = dateutil_parser.parse('11/29/2017 2:00:30pm')
        next_run_time = run_time + datetime.timedelta(seconds=30)
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_last_run'], run_time)
        self.assertEqual(ret['_next_fire_time'], next_run_time)

        time.sleep(2)

        # eval at 2:01:00pm, will run.
        run_time = dateutil_parser.parse('11/29/2017 2:01:00pm')
        next_run_time = run_time + datetime.timedelta(seconds=30)
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_last_run'], run_time)
        self.assertEqual(ret['_next_fire_time'], next_run_time)

        time.sleep(2)

        # eval at 2:01:30pm, will run.
        run_time = dateutil_parser.parse('11/29/2017 2:01:30pm')
        next_run_time = run_time + datetime.timedelta(seconds=30)
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_last_run'], run_time)
        self.assertEqual(ret['_next_fire_time'], next_run_time)

    def test_eval_minutes(self):
        '''
        verify that scheduled job run mutiple times with minutes
        '''
        job_name = 'job_eval_minutes'
        job = {
          'schedule': {
            job_name: {
              'function': 'test.ping',
              'minutes': '30',
            }
          }
        }

        if salt.utils.platform.is_darwin():
            job['schedule'][job_name]['dry_run'] = True

        # Add job to schedule
        self.schedule.opts.update(job)

        # eval at 2:00pm to prime, simulate minion start up.
        run_time = dateutil_parser.parse('11/29/2017 2:00pm')
        next_run_time = run_time + datetime.timedelta(minutes=30)
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_next_fire_time'], next_run_time)

        # eval at 2:00:01pm, will not run.
        run_time = dateutil_parser.parse('11/29/2017 2:00:01pm')
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertNotIn('_last_run', ret)
        self.assertEqual(ret['_next_fire_time'], next_run_time)

        # eval at 2:30:00pm, will run.
        run_time = dateutil_parser.parse('11/29/2017 2:30:00pm')
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_last_run'], run_time)

        time.sleep(2)

        # eval at 3:00:00pm, will run.
        run_time = dateutil_parser.parse('11/29/2017 3:00:00pm')
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_last_run'], run_time)

        time.sleep(2)

        # eval at 3:30:00pm, will run.
        run_time = dateutil_parser.parse('11/29/2017 3:30:00pm')
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_last_run'], run_time)

    def test_eval_hours(self):
        '''
        verify that scheduled job run mutiple times with hours
        '''
        job_name = 'job_eval_hours'
        job = {
          'schedule': {
            job_name: {
              'function': 'test.ping',
              'hours': '2',
            }
          }
        }

        if salt.utils.platform.is_darwin():
            job['schedule'][job_name]['dry_run'] = True

        # Add job to schedule
        self.schedule.opts.update(job)

        # eval at 2:00pm to prime, simulate minion start up.
        run_time = dateutil_parser.parse('11/29/2017 2:00pm')
        next_run_time = run_time + datetime.timedelta(hours=2)
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_next_fire_time'], next_run_time)

        # eval at 2:00:01pm, will not run.
        run_time = dateutil_parser.parse('11/29/2017 2:00:01pm')
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertNotIn('_last_run', ret)
        self.assertEqual(ret['_next_fire_time'], next_run_time)

        # eval at 4:00:00pm, will run.
        run_time = dateutil_parser.parse('11/29/2017 4:00:00pm')
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_last_run'], run_time)

        time.sleep(2)

        # eval at 6:00:00pm, will run.
        run_time = dateutil_parser.parse('11/29/2017 6:00:00pm')
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_last_run'], run_time)

        time.sleep(2)

        # eval at 8:00:00pm, will run.
        run_time = dateutil_parser.parse('11/29/2017 8:00:00pm')
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_last_run'], run_time)

    def test_eval_days(self):
        '''
        verify that scheduled job run mutiple times with days
        '''
        job_name = 'job_eval_days'
        job = {
          'schedule': {
            job_name: {
              'function': 'test.ping',
              'days': '2',
              'dry_run': True
            }
          }
        }

        if salt.utils.platform.is_darwin():
            job['schedule'][job_name]['dry_run'] = True

        # Add job to schedule
        self.schedule.opts.update(job)

        # eval at 11/23/2017 2:00pm to prime, simulate minion start up.
        run_time = dateutil_parser.parse('11/23/2017 2:00pm')
        next_run_time = run_time + datetime.timedelta(days=2)
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_next_fire_time'], next_run_time)

        # eval at 11/25/2017 2:00:00pm, will run.
        run_time = dateutil_parser.parse('11/25/2017 2:00:00pm')
        next_run_time = run_time + datetime.timedelta(days=2)
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_last_run'], run_time)
        self.assertEqual(ret['_next_fire_time'], next_run_time)

        # eval at 11/26/2017 2:00:00pm, will not run.
        run_time = dateutil_parser.parse('11/26/2017 2:00:00pm')
        last_run_time = run_time - datetime.timedelta(days=1)
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_last_run'], last_run_time)
        self.assertEqual(ret['_next_fire_time'], next_run_time)

        time.sleep(2)

        # eval at 11/27/2017 2:00:00pm, will run.
        run_time = dateutil_parser.parse('11/27/2017 2:00:00pm')
        next_run_time = run_time + datetime.timedelta(days=2)
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_last_run'], run_time)
        self.assertEqual(ret['_next_fire_time'], next_run_time)

        time.sleep(2)

        # eval at 11/28/2017 2:00:00pm, will not run.
        run_time = dateutil_parser.parse('11/28/2017 2:00:00pm')
        last_run_time = run_time - datetime.timedelta(days=1)
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_last_run'], last_run_time)
        self.assertEqual(ret['_next_fire_time'], next_run_time)

        time.sleep(2)

        # eval at 11/29/2017 2:00:00pm, will run.
        run_time = dateutil_parser.parse('11/29/2017 2:00:00pm')
        next_run_time = run_time + datetime.timedelta(days=2)
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret['_last_run'], run_time)
        self.assertEqual(ret['_next_fire_time'], next_run_time)

    def test_eval_when_splay(self):
        '''
        verify that scheduled job runs
        '''
        job_name = 'test_eval_when_splay'
        splay = 300
        job = {
          'schedule': {
            job_name: {
              'function': 'test.ping',
              'when': '11/29/2017 4:00pm',
              'splay': splay
            }
          }
        }
        run_time1 = dateutil_parser.parse('11/29/2017 4:00pm')
        run_time2 = run_time1 + datetime.timedelta(seconds=splay)

        # Add the job to the scheduler
        self.schedule.opts.update(job)

        with patch('random.randint', MagicMock(return_value=splay)):
            # Evaluate to prime
            run_time = dateutil_parser.parse('11/29/2017 3:00pm')
            self.schedule.eval(now=run_time)
            ret = self.schedule.job_status(job_name)

            # Evaluate at expected runtime1, should not run
            self.schedule.eval(now=run_time1)
            ret = self.schedule.job_status(job_name)
            self.assertNotIn('_last_run', ret)

            # Evaluate at expected runtime2, should run
            self.schedule.eval(now=run_time2)
            ret = self.schedule.job_status(job_name)
            self.assertEqual(ret['_last_run'], run_time2)

    def test_eval_when_splay_in_past(self):
        '''
        verify that scheduled job runs
        '''
        job_name = 'test_eval_when_splay_in_past'
        splay = 300
        job = {
          'schedule': {
            job_name: {
              'function': 'test.ping',
              'when': ['11/29/2017 6:00am'],
              'splay': splay
            }
          }
        }
        run_time1 = dateutil_parser.parse('11/29/2017 4:00pm')

        # Add the job to the scheduler
        self.schedule.opts.update(job)

        # Evaluate to prime
        run_time = dateutil_parser.parse('11/29/2017 3:00pm')
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)

        # Evaluate at expected runtime1, should not run
        # and _next_fire_time should be None
        self.schedule.eval(now=run_time1)
        ret = self.schedule.job_status(job_name)
        self.assertNotIn('_last_run', ret)
        self.assertEqual(ret['_next_fire_time'], None)
